@@ -1,7 +1,23 @@
 #!/usr/bin/python
-# Copyright (c) 2003-2004 Hyriand. All rights reserved.
+# Copyright (C) 2007 daelstorm. All rights reserved.
 #
-indent = "        "
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+# Original copyright below
+# Copyright (c) 2003-2004 Hyriand. All rights reserved.
+
+indent = "\t"
 class Widget:
 	def __init__(self, my_class, id):
 		self.my_class = my_class
@@ -16,14 +32,15 @@ class Widget:
 attrs = [
 	["title", lambda w, v: "%s.set_title(_(\"%s\"))" %(w, v.replace("\"", "\\\""))],
 	["active", lambda w, v: "%s.set_active(%s)" % (w, v.capitalize())],
-	["label", lambda w, v: "%s.set_label(_(\"%s\"))" %(w, v.replace("\"", "\\\""))],
+	["label", lambda w, v: "%s.set_label(_(\"%s\"))" %(w, v.replace("\"", "\\\"").replace("\n", "\\n"))],
 	["window_position", lambda w,v: "%s.set_position(%s)" % (w, v.replace("GTK_", "gtk."))],
 	["tab_pos", lambda w,v: "%s.set_tab_pos(%s)" % (w, v.replace("GTK_", "gtk."))],
 	["has_resize_grip", lambda w,v: "%s.set_has_resize_grip(%s)" % (w,v.capitalize())],
-	["text", lambda w,v: "%s.set_text(\"%s\")" % (w, v.replace("\"", "\\\""))],
+	["text", lambda w,v: ("%s.set_text(_(\"%s\"))" % (w, v.replace("\"", "\\\""))).replace("_(\"\")", "\"\"")],
 	["accel_group", lambda w,v: "%s.add_accel_group(%s)" % (w, v)],
 	["wrap_mode", lambda w,v: "%s.set_wrap_mode(%s)" % (w, v.replace("GTK_", "gtk."))],
 	["wrap", lambda w,v: "%s.set_line_wrap(%s)" % (w, v.capitalize())],
+	["set_markup", lambda w,v: "%s.set_markup(_(\"%s\"))" % (w, v)],
 	["cursor_visible", lambda w,v: "%s.set_cursor_visible(%s)" % (w, v.capitalize())],
 	["editable", lambda w,v: "%s.set_editable(%s)" % (w, v.capitalize())],
 	["scrollable", lambda w,v: "%s.set_scrollable(%s)" % (w, v.capitalize())],
@@ -32,14 +49,17 @@ attrs = [
 	["visible", lambda w, v: (v == "True" and ("%s.show()" % w)) or ""],
 	["visibility", lambda w, v: (v == "True" and ("%s.set_visibility(True)" % w)) or ("%s.set_visibility(False)" % w)],	
 	["headers_visible", lambda w, v: "%s.set_headers_visible(%s)" % (w, v.capitalize())],
+	["sensitive", lambda w,v: "%s.set_sensitive(%s)" % (w,v)],
 	["image", lambda w,v: "%s.set_image(%s)" % (w,v)],
+	["digits", lambda w,v: "%s.set_digits(%s)" % (w,v)],
 	["spacing", lambda w,v: "%s.set_spacing(%s)" % (w,v)],
 	["border_width", lambda w,v: "%s.set_border_width(%s)" % (w,v)],
+	["width_chars", lambda w,v: "%s.set_width_chars(%s)" % (w,v)],
 	["row_spacing", lambda w,v: "%s.set_row_spacings(%s)" % (w,v)],
 	["column_spacing", lambda w,v: "%s.set_col_spacings(%s)" % (w,v)],
 	["layout_style", lambda w,v: "%s.set_layout(%s)" % (w, v.replace("GTK_", "gtk."))],
 	["shadow_type", lambda w,v: "%s.set_shadow_type(%s)" % (w, v.replace("GTK_", "gtk."))],
-	["items", lambda w,v: "%s_List.append([\"%s\"])" % (w, v.replace("\"", "\\\""))],
+	["items", lambda w,v: "for i in [_(\"%s\")]:\n%s\t%s_List.append([i])" % ("\"), _(\"".join(v.replace("\"", "\\\"").split("\n")), indent, w)],
 ]
 
 PM_NONE = 0
@@ -69,7 +89,22 @@ def write_widget_attrs(widget):
 			y = widget.attrs["yalign"]
 		if float(x) != 0.5 or float(y) != 0.5:
 			print indent + "%s.set_alignment(%s, %s)" % (widget.id, x, y)
-	
+	top_padding = bottom_padding = left_padding = right_padding = 0
+	if widget.attrs.has_key("top_padding"):
+		top_padding = widget.attrs["top_padding"]
+	if widget.attrs.has_key("bottom_padding"):
+		bottom_padding = widget.attrs["bottom_padding"]
+	if widget.attrs.has_key("left_padding"):
+		left_padding = widget.attrs["left_padding"]
+	if widget.attrs.has_key("right_padding"):
+		right_padding = widget.attrs["right_padding"]
+	for i in [top_padding, bottom_padding, left_padding, right_padding]:
+		if i != 0:
+			print indent + "%s.set_padding(%s, %s, %s, %s)" % (widget.id, top_padding, bottom_padding, left_padding, right_padding)
+			break
+			
+	if widget.attrs.has_key("angle"):
+		print indent + "%s.set_angle(%s)" % (widget.id, widget.attrs["angle"])
 	w, h = "-1", "-1"
 	if widget.attrs.has_key("width_request"):
 		w = widget.attrs["width_request"]
@@ -77,19 +112,37 @@ def write_widget_attrs(widget):
 		h = widget.attrs["height_request"]
 	if not (w == "-1" and h == "-1"):
 		print indent + "%s.set_size_request(%s, %s)" % (widget.id, w, h)
-
+	x = y = 0
 	if widget.attrs.has_key("xpad"):
 		x = widget.attrs["xpad"]
+	if widget.attrs.has_key("ypad"):
 		y = widget.attrs["ypad"]
+	if x != 0 or y != 0:
 		print indent + "%s.set_padding(%s, %s)" % (widget.id, x, y)
 		
 	if widget.attrs.has_key("stock"):
 		img = "gtk.STOCK_" + widget.attrs["stock"][4:].upper().replace("-", "_")
-		sze = widget.attrs["icon_size"]
+		if widget.attrs.has_key("icon_size"):
+			sze = widget.attrs["icon_size"]
+		else:
+			sze = 4
 		print indent + "%s.set_from_stock(%s, %s)" % (widget.id, img, sze)
-	
+		
+	if widget.attrs.has_key("tooltip"):
+		tip = widget.attrs["tooltip"]
+		print indent + "self.tooltips.set_tip(%s, _(\"%s\"))" % (widget.id, tip.replace("\"", "\\\""))
+		
+	if widget.attrs.has_key("expanded"):
+		expanded = widget.attrs["expanded"]
+		print indent + "%s.set_expanded(%s)" % (widget.id, expanded)
+		
+		
 	for i in attrs:
 		if widget.attrs.has_key(i[0]):
+			#print widget.attrs[i[0]]
+			if i[0] == "width_chars" and widget.attrs[i[0]] == "-1":
+				continue
+				
 			v = i[1](widget.id, widget.attrs[i[0]])
 			if v:
 				print indent + "%s" % v
@@ -101,19 +154,82 @@ def write_widget_attrs(widget):
 			signals.append(callback)
 	
 	for accel in widget.accelerators:
-		print indent + "%s.add_accelerator(\"%s\", self.accel_group, gtk.gdk.keyval_from_name(\"%s\"), %s, gtk.ACCEL_VISIBLE)" % (widget.id, accel[2], accel[0], accel[1])
+		key, modifer, signal = accel[0], accel[1], accel[2]
+		if modifer == "" or modifer.isspace():
+			modifer = "0"
+		print indent + "%s.add_accelerator(\"%s\", self.accel_group, gtk.gdk.keyval_from_name(\"%s\"), %s, gtk.ACCEL_VISIBLE)" % (widget.id, signal, key, modifer)
 		
+def write_widget_adjustment(widget, my_class, *args):
+	restargs = ""
+	value, min, max, step_incr, page_incr, page_size = widget.attrs["adjustment"].split(" ")
+	print value, min, max, step_incr, page_incr, page_size
+	print indent + "%s_adj = gtk.Adjustment(value=%s, lower=%s, upper=%s, step_incr=%s, page_incr=%s, page_size=%s)" % (widget.id, value, min, max, step_incr, page_incr, page_size)
+
+def write_widget_spinbutton(widget, my_class, *args):
+	global signals, indent
+	#{'digits': '0', 'climb_rate': '1', 'update_policy': 'GTK_UPDATE_ALWAYS', 'can_focus': 'True', 'numeric': 'False', 'visible': 'True', 'snap_to_ticks': 'False', 'wrap': 'False', 'adjustment': '0 0 255 1 10 10'}
+	value, min, max, step_incr, page_incr, page_size = widget.attrs["adjustment"].split(" ")
+	adjustment = "gtk.Adjustment(value=%s, lower=%s, upper=%s, step_incr=%s, page_incr=%s, page_size=%s)" % (value, min, max, step_incr, page_incr, page_size)
+	restargs = ""
+	print indent + "%s = gtk.%s(%s)" % (widget.id, my_class, adjustment)
+	allowed = ["visible", "width_chars"]
+	for i in attrs:
+		if i[0] in allowed:
+			if widget.attrs.has_key(i[0]):
+				if i[0] == "width_chars" and widget.attrs["width_chars"] == -1:
+					continue
+				v = i[1](widget.id, widget.attrs[i[0]])
+				if v:
+					print indent + "%s" % v
+	for signal in widget.signals.keys():
+		callback = widget.signals[signal]
+		print indent + "%s.connect(\"%s\", self.%s)" % (widget.id, signal, callback)
+		if not callback in signals:
+			signals.append(callback)
+	print 
+def write_widget_scale(widget, my_class, *args):
+	global signals, indent
+	restargs = ""
+	
+	value, min, max, step_incr, page_incr, page_size = widget.attrs["adjustment"].split(" ")
+	adjustment = "gtk.Adjustment(value=%s, lower=%s, upper=%s, step_incr=%s, page_incr=%s, page_size=%s)" % (value, min, max, step_incr, page_incr, page_size)
+	restargs = ""
+	print indent + "%s = gtk.%s(%s)" % (widget.id, my_class, adjustment)
+
+	write_widget_attrs(widget)
+	print 
+	
 def write_widget_generic(widget, my_class, *args):
 	global signals, indent
 	restargs = ""
 	for arg in args[0:]:
-		if arg[0] == "@":
-			try:
+		
+		if arg[0] == "+":
+			name = arg[1:]
+			if widget.attrs.has_key(name):
 				arg = widget.attrs[arg[1:]]
-			except:
-				arg= "None"
+			else:
+				arg= ""
+		elif arg[0] == "@":
+			name = arg[1:]
+
+			if widget.attrs.has_key(name):
+				arg = widget.attrs[name]
+			if name in ("xalign", "yalign") and not widget.attrs.has_key(name):
+				arg = "0.5"
+				#pass
+			else:
+				arg= "0"
 		elif arg[0] == "$":
-			s = widget.attrs[arg[1:]].replace("\"", "\\\"")
+			
+			if not arg[1:] in widget.attrs.keys():
+				continue
+			s = widget.attrs[arg[1:]].replace("\"", "\\\"").replace("\n", "\\n")
+			
+			if arg[1:] == "label":
+				if widget.attrs.has_key("use_markup") and widget.attrs["use_markup"] == "True":
+					widget.attrs["set_markup"] = s
+					s = ""
 			if s:
 				narg = '_("%s")' % s
 			else:
@@ -121,7 +237,10 @@ def write_widget_generic(widget, my_class, *args):
 			del widget.attrs[arg[1:]]
 			arg = narg
 		elif arg[0] == "#":
-			arg = "%s" % widget.attrs[arg[1:]].capitalize()
+			name = arg[1:]
+			if widget.attrs.has_key(name):
+				arg = "%s" % (widget.attrs[name].capitalize())
+			else: arg = "False"
 		if arg[:4] == "GTK_":
 			arg = "gtk." + arg[4:]
 		if restargs:
@@ -129,7 +248,7 @@ def write_widget_generic(widget, my_class, *args):
 		restargs += arg
 	print indent + "%s = gtk.%s(%s)" % (widget.id, my_class, restargs)
 	write_widget_attrs(widget)
-	indent = "        "
+	indent = "\t\t"
 	
 def write_widget_container(widget, my_class, pack, *args):
 	write_widget_generic(widget, my_class, *args)
@@ -141,8 +260,15 @@ def write_widget_container(widget, my_class, pack, *args):
 		if pack == PM_PACK:
 			if w.packing.has_key("expand"):
 				x = w.packing["expand"].capitalize()
-				f = w.packing["fill"].capitalize()
-				p = w.packing["padding"]
+				if w.packing.has_key("fill"):
+					f = w.packing["fill"].capitalize()
+				else:
+					f = "True"
+				if w.packing.has_key("padding"):
+					p = w.packing["padding"]
+				else:
+					p = "0"
+					
 				if w.packing.has_key("pack_type"):
 					packtype = w.packing["pack_type"]
 				else:
@@ -155,8 +281,14 @@ def write_widget_container(widget, my_class, pack, *args):
 			else:
 				print indent + "%s.pack_start(%s)" % (widget.id, w.id)
 		elif pack == PM_PACK12:
-			r = w.packing["resize"].capitalize()
-			s = w.packing["shrink"].capitalize()
+			if w.packing.has_key("resize"):
+				r = w.packing["resize"].capitalize()
+			else:
+				r = "True"
+			if w.packing.has_key("shrink"):	
+				s = w.packing["shrink"].capitalize()
+			else:
+				s = "True"
 			i = widget.children.index(w) + 1
 			print indent + "%s.pack%i(%s, %s, %s)" % (widget.id, i, w.id, r, s)
 		elif pack == PM_ADD:
@@ -168,23 +300,36 @@ def write_widget_container(widget, my_class, pack, *args):
 			else:
 				print indent + "%s.add(%s)" % (widget.id, w.id)
 		elif pack == PM_ATTACH:
-			la = w.packing["left_attach"]
-			ra = w.packing["right_attach"]
-			ta = w.packing["top_attach"]
-			ba = w.packing["bottom_attach"]
+			if not w.packing.has_key("left_attach"):
+				la = 0
+			else:
+				la = w.packing["left_attach"]
+			if not w.packing.has_key("right_attach"):
+				ra = 1
+			else:
+				ra = w.packing["right_attach"]
+			#print w.packing, pack
+			if not w.packing.has_key("top_attach"):
+				ta = 0
+			else:
+				ta = w.packing["top_attach"]
+			if not w.packing.has_key("bottom_attach"):
+				ba = 1
+			else:
+				ba = w.packing["bottom_attach"]
 			xopts = "gtk.EXPAND|gtk.FILL"
 			yopts = "gtk.EXPAND|gtk.FILL"
 			if w.packing.has_key("x_options"):
 				if w.packing["x_options"] == "":
 					xopts = "0"
-				elif w.packing["x_options"] == "fill":
+				elif w.packing["x_options"] in ("fill", "GTK_FILL"):
 					xopts = "gtk.FILL"
 				elif w.packing["x_options"] == "expand":
 					xopts = "gtk.EXPAND"
 			if w.packing.has_key("y_options"):
 				if w.packing["y_options"] == "":
 					yopts = "0"
-				elif w.packing["y_options"] == "fill":
+				elif w.packing["y_options"] in ("fill", "GTK_FILL"):
 					yopts = "gtk.FILL"
 				elif w.packing["y_options"] == "expand":
 					yopts = "gtk.EXPAND"
@@ -192,6 +337,8 @@ def write_widget_container(widget, my_class, pack, *args):
 		print
 	for wid in widget.internalchildren.keys():
 		w = widget.internalchildren[wid]
+		if my_class == "ComboBoxEntry":
+			wid = "child"
 		print indent + "%s = %s.%s" % (w.id, widget.id, wid)
 		write_widget_attrs(w)
 		print
@@ -222,7 +369,34 @@ def write_widget_menuitem(widget, my_class):
 		write_widget(w)
 		print indent + "%s.set_submenu(%s)" % (widget.id, w.id)
 		print
+
+def write_widget_radiomenuitem(widget, my_class):
+	args = []
+	group = "None"
+	if widget.attrs.has_key("group"):
+		group = "self.%s" % widget.attrs["group"]
+		del widget.attrs["group"]
+	args.append(group)
+	
+	if widget.attrs.has_key("label"):
+		label = widget.attrs["label"].replace("\"", "\\\"")
+		if label:
+			args.append("_(\"%s\")" % label)
+		else:
+			args.append('""')
+
+		del widget.attrs["label"]
+	else:
+		label = ""
+	write_widget_generic(widget, my_class, *args)
+	print
+	if widget.children:
+		w = widget.children[0]
+		write_widget(w)
+		print indent + "%s.set_submenu(%s)" % (widget.id, w.id)
+		print
 		
+
 def write_widget_imagemenuitem(widget, my_class):
 	args = []
 	if widget.attrs.has_key("label"):
@@ -287,6 +461,15 @@ def write_widget_button(widget):
 			return
 	write_widget_container(widget, "Button", PM_ADD)
 
+def write_widget_togglebutton(widget):
+	if widget.attrs.has_key("use_stock"):
+		if widget.attrs["use_stock"] == "True":
+			stock = "gtk.STOCK_" + widget.attrs["label"][4:].upper().replace("-", "_")
+			del widget.attrs["label"]
+			write_widget_container(widget, "ToggleButton", PM_ADD, "None", stock)
+			return
+	write_widget_container(widget, "ToggleButton", PM_ADD)
+	
 def write_widget_combobox(widget):
 	print indent + "%s_List = gtk.ListStore(gobject.TYPE_STRING)" % (widget.id)
 	write_widget_container(widget, "ComboBox", PM_ADD)
@@ -312,7 +495,7 @@ def write_widget_custom(widget):
 	if widget.attrs.has_key("int2"):
 		i2 = widget.attrs["int2"]
 	if s1:
-		s1 = "_(\"%s\")" % s1
+		s1 = "\"%s\"" % s1
 	else:
 		s1 = '""'
 	if s2:
@@ -323,7 +506,9 @@ def write_widget_custom(widget):
 	write_widget_attrs(widget)
 
 classes = {
-	"GtkWindow": [write_widget_container, "Window", PM_NONE, "@type"],
+	"GtkWindow": [write_widget_container, "Window", PM_NONE, "+type"],
+	"GtkDialog": [write_widget_container, "Dialog", PM_NONE],
+	"GtkAboutDialog": [write_widget_container, "AboutDialog", PM_NONE],
 	"GtkOptionMenu": [write_widget_container, "OptionMenu", PM_NONE],
 	"GtkVBox": [write_widget_container, "VBox", PM_PACK, "#homogeneous", "@spacing"],
 	"GtkHBox": [write_widget_container, "HBox", PM_PACK, "#homogeneous", "@spacing"],
@@ -331,6 +516,7 @@ classes = {
 	"GtkMenuItem": [write_widget_menuitem, "MenuItem"],
 	"GtkImageMenuItem": [write_widget_imagemenuitem, "ImageMenuItem"],
 	"GtkSeparatorMenuItem": [write_widget_menuitem, "MenuItem"],
+	"GtkRadioMenuItem": [write_widget_radiomenuitem, "RadioMenuItem"],
 	"GtkCheckMenuItem": [write_widget_menuitem, "CheckMenuItem"],
 	"GtkMenu": [write_widget_menu, "Menu"],
 	"GtkVPaned": [write_widget_container, "VPaned", PM_PACK12],
@@ -344,11 +530,18 @@ classes = {
 	"GtkComboBox": [write_widget_combobox],
 	"GtkComboBoxEntry": [write_widget_comboboxentry],
 	"GtkComboBoxEntryChild": [write_widget_generic, "child"],
-	"GtkCheckButton": [write_widget_generic, "CheckButton"],
+	"GtkCheckButton": [write_widget_container, "CheckButton", PM_ADD],
+	"GtkToggleButton": [write_widget_togglebutton],
 	"GtkFontButton": [write_widget_generic, "FontButton"],
+	"GtkIconView": [write_widget_generic, "IconView"],
+	"GtkFileChooserButton": [write_widget_generic, "FileChooserButton"],
 	"GtkRadioButton": [write_widget_radiobutton],
 	"GtkButton": [write_widget_button],
 	"GtkTextView": [write_widget_textview],
+	"GtkSpinButton": [write_widget_spinbutton, "SpinButton"],
+	"GtkHScale": [write_widget_scale, "HScale"],
+	"GtkVScale": [write_widget_scale, "VScale"],
+	"GtkAdjustment": [write_widget_adjustment, "Adjustment"],
 	"GtkStatusbar": [write_widget_generic, "Statusbar"],
 	"GtkEntry": [write_widget_generic, "Entry"],
 	"GtkList": [write_widget_generic, "List"],
@@ -358,7 +551,9 @@ classes = {
 	"GtkProgressBar": [write_widget_generic, "ProgressBar"],
 	"GtkImage": [write_widget_generic, "Image"],
 	"GtkVSeparator": [write_widget_generic, "VSeparator"],
+	"GtkHSeparator": [write_widget_generic, "HSeparator"],
 	"GtkHButtonBox": [write_widget_container, "HButtonBox", PM_PACK],
+	"GtkEventBox" : [write_widget_container, "EventBox", PM_ADD],
 	"GtkAlignment": [write_widget_container, "Alignment", PM_ADD, "@xalign", "@yalign", "@xscale", "@yscale"],
 	"Custom": [write_widget_custom],
 }
@@ -368,6 +563,7 @@ def write_widget(w):
 		return
 	if not w.my_class in classes:
 		sys.stderr.write("oops... widget class %s not found!\n" % w.my_class)
+		print indent + "# %s of %s should be here" % (w.id, w.my_class)
 		return
 	c = classes[w.my_class]
 	c[0](w, *c[1:])
@@ -412,28 +608,35 @@ def process_accelerator(child):
 		
 def write_main(w):
 	global signals, indent
+	indent = "\t"
 	w.attrs["accel_group"] = "self.accel_group"
 	print "class %s:" % w.id[5:]
-	print "    def __init__(self, create = True, accel_group = None):"
-	print "        if accel_group is None:"
-	print "             self.accel_group = gtk.AccelGroup()"
-	print "        else:"
-	print "             self.accel_group = accel_group"
-	print "        if create:"
-	indent = "            "
+	print indent + "def __init__(self, create = True, accel_group = None, tooltips = None):"
+	print indent + indent + "if accel_group is None:"
+	print indent + indent + indent + "self.accel_group = gtk.AccelGroup()"
+	print indent + indent + "else:"
+	print indent + indent + indent + "self.accel_group = accel_group"
+	print indent + indent + "if tooltips is None:"
+	print indent + indent + indent + "self.tooltips = gtk.Tooltips()"
+	print indent + indent + "else:"
+	print indent + indent + indent + "self.tooltips = tooltips"
+	print indent + indent + "self.tooltips.enable()"
+	print indent + indent + "if create:"
+	indent = "\t\t\t"
 	write_widget(w)
+	indent = "\t"
 	if w.my_class == "GtkWindow":
-		print "        if create:"
-		print "            %s.add(%s)" % (w.id, w.children[0].id)
+		print indent + indent + "if create:"
+		print indent + indent + indent + "%s.add(%s)" % (w.id, w.children[0].id)
 	print
 	for s in signals:
-		print "    def %s(self, widget):" % s
-		print "        pass"
+		print indent + "def %s(self, widget):" % s
+		print indent + indent + "pass"
 		print
 	signals = []
-	print "    def get_custom_widget(self, id, string1, string2, int1, int2):"
-	print "        w = gtk.Label(_(\"(custom widget: %s)\") % id)"
-	print "        return w"
+	print indent + "def get_custom_widget(self, id, string1, string2, int1, int2):"
+	print indent + indent + "w = gtk.Label(_(\"(custom widget: %s)\") % id)"
+	print indent + indent + "return w"
 	print
 
 def process_widget(widget):
