@@ -1,8 +1,8 @@
-# Copyright (C) 2006 daelstorm <daelstorm@gmail.com>
+# Copyright (C) 2007 daelstorm. All rights reserved.
 #
-# This program is free software; you can redistribute it and/or modify
+# This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
@@ -11,30 +11,34 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import gtk, gobject
-import os, commands, sys, re
+import os, commands, sys, re, thread, threading
 from pynicotine.utils import _
 
 class NowPlaying:
-	def __init__(self, frame, create = True, accel_group = None):
+	def __init__(self, frame):
 		self.frame = frame
-		if accel_group is None:
-			self.accel_group = gtk.AccelGroup()
-		else:
-			self.accel_group = accel_group
-		if create:
-			self.NowPlaying = gtk.Window(gtk.WINDOW_TOPLEVEL)
-			self.NowPlaying.set_title(_("NowPlaying"))
-			self.NowPlaying.set_position(gtk.WIN_POS_NONE)
-			self.NowPlaying.add_accel_group(self.accel_group)
-			#self.NowPlaying.show()
-			self.NowPlaying.connect("destroy", self.quit)
-			self.NowPlaying.connect("destroy-event", self.quit)
-			self.NowPlaying.connect("delete-event", self.quit)
+		self.accel_group = gtk.AccelGroup()
+		self.NowPlaying = gtk.Window(gtk.WINDOW_TOPLEVEL)
+		self.NowPlaying.set_title(_("Nicotine+: Configure Now Playing"))
+		self.NowPlaying.set_icon(self.frame.images["n"])
+		self.NowPlaying.set_position(gtk.WIN_POS_NONE)
+		self.NowPlaying.set_modal(True)
+		self.NowPlaying.set_transient_for(self.frame.MainWindow)
+		self.NowPlaying.add_accel_group(self.accel_group)
 	
+		self.NowPlaying.connect("destroy", self.quit)
+		self.NowPlaying.connect("destroy-event", self.quit)
+		self.NowPlaying.connect("delete-event", self.quit)
+		try:
+			import dbus
+			import dbus.glib
+			self.bus = dbus.SessionBus()
+		except Exception, e:
+			self.bus = None
+		self.NowPlaying.set_resizable(False)
 		self.vbox1 = gtk.VBox(False, 5)
 		self.vbox1.show()
 		self.vbox1.set_spacing(5)
@@ -84,7 +88,7 @@ class NowPlaying:
 		self.NP_mpd.connect("clicked", self.OnNPPlayer)
 	
 		self.hbox1.pack_start(self.NP_mpd, False, False, 0)
-	
+
 		self.vbox1.pack_start(self.hbox1, False, True, 0)
 	
 		self.hbox2 = gtk.HBox(False, 5)
@@ -116,9 +120,16 @@ class NowPlaying:
 		self.NP_rhythmbox.show()
 		self.NP_rhythmbox.connect("clicked", self.OnNPPlayer)
 		
-	
 		self.hbox2.pack_start(self.NP_rhythmbox, False, False, 0)
+		
+		self.NP_exaile = gtk.RadioButton(self.NP_infopipe)
+		self.NP_exaile.set_active(False)
+		self.NP_exaile.set_label(_("Exaile"))
+		self.NP_exaile.show()
+		self.NP_exaile.connect("clicked", self.OnNPPlayer)
 	
+		self.hbox2.pack_start(self.NP_exaile, False, False, 0)
+			
 		self.NP_other = gtk.RadioButton(self.NP_infopipe)
 		self.NP_other.set_active(False)
 		self.NP_other.set_label(_("Other"))
@@ -188,9 +199,10 @@ class NowPlaying:
 		self.NPFormat.set_model(self.NPFormat_List)
 		self.NPFormat.set_text_column(0)
 		self.NPFormat.child.connect("activate", self.OnAddFormat)
+		self.NPFormat.child.connect("changed", self.OnModifyFormat)
 		self.vbox1.set_border_width(5)
 		self.vbox1.pack_start(self.NPFormat, False, False, 0)
-		
+
 		self.hbox6 = gtk.HBox(False, 5)
 		self.hbox6.show()
 		self.hbox6.set_spacing(5)
@@ -258,28 +270,45 @@ class NowPlaying:
 		self.vbox1.pack_start(self.hbox3, False, True, 0)
 		self.defaultlist = [ "$n", "$a - $t", "[$a] $t", "Now $s: [$a] $t", "Now $s: $n", "$a - $b - $t", "$a - $b - $t ($l/$rKBps) from $y $c" ]
 		self.title_clear()
-		
+		self.player_replacers = []
 		self.OnNPPlayer(None)
 	
-		if create:
-			self.NowPlaying.add(self.vbox1)
+		self.NowPlaying.add(self.vbox1)
 		
-		if self.frame.np.config.sections["players"]["npformat"] != "":
-			self.NPFormat.child.set_text(self.frame.np.config.sections["players"]["npformat"])
-		if self.frame.np.config.sections["players"]["npformatlist"] != []:
-			if self.frame.np.config.sections["players"]["npformat"] == "":
-				self.NPFormat.child.set_text(str(self.frame.np.config.sections["players"]["npformatlist"][0]))
-			for item in self.frame.np.config.sections["players"]["npformatlist"]:
+		# Set the active radio button
+		config = self.frame.np.config.sections
+		self.SetPlayer(config["players"]["npplayer"])
+		if config["players"]["npformat"] != "":
+			self.NPFormat.child.set_text(config["players"]["npformat"])
+		if config["players"]["npformatlist"] != []:
+			if config["players"]["npformat"] == "":
+				self.NPFormat.child.set_text(str(config["players"]["npformatlist"][0]))
+			for item in config["players"]["npformatlist"]:
 				self.NPFormat_List.append([item])
-		if self.frame.np.config.sections["players"]["npformat"] == "":
+		if config["players"]["npformat"] == "":
 			self.NPFormat.child.set_text(str(self.defaultlist[0]))
-
+		self.NPCommand.set_text(config["players"]["npothercommand"])
 		
 		for item in self.defaultlist:
 			self.NPFormat_List.append([str(item)])
-		# Set the active radio button
-		self.SetPlayer(self.frame.np.config.sections["players"]["npplayer"])
 		
+		
+	def SetTextBG(self, widget, bgcolor="", fgcolor=""):
+		if bgcolor == "":
+			colour = None
+		else:
+			colour = gtk.gdk.color_parse(bgcolor)
+		widget.modify_base(gtk.STATE_NORMAL, colour)
+		widget.modify_bg(gtk.STATE_NORMAL, colour)
+
+		if type(widget) in (gtk.Entry, gtk.SpinButton):
+			if fgcolor == "":
+				colour = None
+			else:
+				colour = gtk.gdk.color_parse(fgcolor)
+			widget.modify_text(gtk.STATE_NORMAL, colour)
+			widget.modify_fg(gtk.STATE_NORMAL, colour)
+
 	def title_clear(self):
 		self.label7.set_text("")
 		self.title = { "title": "", "artist": "", "comment": "", "year": "", "album": "", "track":"", "length": "", "nowplaying": "", "status": "", "bitrate": "", "filename": ""}
@@ -299,8 +328,25 @@ class NowPlaying:
 			self.NP_rhythmbox.set_active(1)
 		elif player == "bmpx":
 			self.NP_bmpx.set_active(1)
+		elif player == "exaile":
+			self.NP_exaile.set_active(1)
 		elif player == "other":
 			self.NP_other.set_active(1)
+			self.player_replacers = ["$n"]
+		else:
+			self.NP_other.set_active(1)
+			
+	def OnModifyFormat(self, widget):
+		text = self.NPFormat.child.get_text().strip()
+		replacers = []
+		for replacer in ["$n", "$t", "$l", "$a", "$b", "$c", "$k", "$y", "$r", "$f", "$s"]:
+			if replacer in text:
+				replacers.append(replacer)
+		for replacer in replacers:
+			if replacer not in self.player_replacers:
+				self.frame.SetTextBG(self.NPFormat.child, "red", "white")
+				return
+		self.frame.SetTextBG(self.NPFormat.child, "", "")
 			
 	def OnAddFormat(self, widget):
 		text = self.NPFormat.child.get_text().strip()
@@ -328,31 +374,62 @@ class NowPlaying:
 	def OnNPPlayer(self, widget):
 		set = 0 
 		if self.NP_infopipe.get_active():
-			self.label2.set_text(_("$n Now Playing\n$l Length\n$r Bitrate\n$s Status"))
+			self.player_replacers = ["$n",  "$l", "$b", "$c", "$k", "$r", "$f", "$s"]
 			set = 1
-		if self.NP_mpd.get_active():
-			self.label2.set_text(_("$n Now Playing\n$t Title\n$a Artist\n$b Album\n$k Track Number\n$f Filename"))
+		elif self.NP_mpd.get_active():
+			self.player_replacers = ["$n", "$t", "$a", "$b",  "$f", "$k"]
 			set = 1
-		if self.NP_amarok.get_active():
-			self.label2.set_text(_("$n Now Playing\n$l Length\n$r Bitrate\n$s Status\n$t Title\n$a Artist\n$b Album\n$c Comment\n$k Track Number\n$y Year\n$f Filename"))
+		elif self.NP_amarok.get_active():
+			self.player_replacers = ["$n", "$t", "$l", "$a", "$b", "$c", "$k", "$y", "$r", "$f", "$s"]
 			set = 1
-		if self.NP_audacious.get_active():
-			self.label2.set_text(_("$n Now Playing\n$l Length\n$r Bitrate\n$s Status\n$t Title\n$a Artist\n$b Album\n$c Comment\n$k Track Number\n$y Year\n$f Filename"))
+		elif self.NP_audacious.get_active():
+			self.player_replacers = ["$n", "$t", "$l", "$a", "$b", "$c", "$k", "$y", "$r", "$f", "$s"]
 			set = 1
-		if self.NP_rhythmbox.get_active():
-			self.label2.set_text(_("$n Now Playing\n$l Length\n$r Bitrate\n$s Status\n$t Title\n$a Artist\n$b Album\n$c Comment\n$k Track Number\n$y Year\n$f Filename (URI)"))
+		elif self.NP_rhythmbox.get_active():
+			self.player_replacers = ["$n", "$t", "$l", "$a", "$b", "$c", "$k", "$y", "$r", "$f", "$s"]
 			set = 1
-		if self.NP_bmpx.get_active():
-			self.label2.set_text(_("$n Now Playing\n$l Length\n$r Bitrate\n$t Title\n$a Artist\n$b Album\n$k Track Number\n$y Year\n$f Filename (URI)"))
+		elif self.NP_bmpx.get_active():
+			self.player_replacers = ["$n", "$t", "$l", "$a", "$b", "$k", "$y", "$r", "$f"]
+			set = 1
+		elif self.NP_exaile.get_active():
+			self.player_replacers = ["$t", "$l", "$a", "$b"]
 			set = 1
 		if self.NP_other.get_active():
-			self.label2.set_text(_("$n Now Playing"))
+			self.player_replacers = ["$n"]
 			self.NPCommand.set_sensitive(True)
 			set = 1
 		else:
 			self.NPCommand.set_sensitive(False)
+		legend = ""
+		for item in self.player_replacers:
+			legend += item + " "
+			if item == "$t":
+				legend += _("Title")
+			elif item == "$n":
+				legend += _("Now Playing (typically \"%s - %s\")") % (_("Artist"), _("Title"))
+			elif item == "$l":
+				legend += _("Length")
+			elif item == "$r":
+				legend += _("Bitrate")
+			elif item == "$c":
+				legend += _("Comment")
+			elif item == "$a":
+				legend += _("Artist")
+			elif item == "$b":
+				legend += _("Album")
+			elif item == "$k":
+				legend += _("Track Number")
+			elif item == "$y":
+				legend += _("Year")
+			elif item == "$f":
+				legend += _("Filename (URI)")
+			elif item == "$s":
+				legend += _("Status")
+			legend += "\n"
+		self.label2.set_text(legend)
 		if not set:
 			self.label2.set_text("")
+		self.OnModifyFormat(self.NPFormat.child)
 			
 	def OnNPCancel(self, widget):
 		self.quit(None)
@@ -362,33 +439,19 @@ class NowPlaying:
 		return True
 		
 	def OnNPTest(self, widget):
-		self.DisplayNowPlaying(None, test=1)
+
+		self.DisplayNowPlaying(None, 1)
 		
-	def OnNPSave(self, widget):
-		if self.NP_infopipe.get_active():
-			player = "infopipe"
-		elif self.NP_amarok.get_active():
-			player = "amarok"
-		elif self.NP_audacious.get_active():
-			player = "audacious"
-		elif self.NP_mpd.get_active():
-			player = "mpd"
-		#elif self.NP_mp3blaster.get_active():
-		#	player = "mp3blaster"
-		elif self.NP_rhythmbox.get_active():
-			player = "rhythmbox"
-		elif self.NP_bmpx.get_active():
-			player = "bmpx"
-		elif self.NP_other.get_active():
-			player = "other"
+	def DisplayNowPlaying(self, widget, test=0, callback=None):
+
+		if self.NP_rhythmbox.get_active() or self.NP_bmpx.get_active():
+			# dbus (no threads, please)
+			self.GetNP(None, test, callback)
+		else:
+			# thread (command execution)
+			thread.start_new_thread(self.GetNP, (None, test, callback))
 			
-		self.frame.np.config.sections["players"]["npplayer"] = player
-		self.frame.np.config.sections["players"]["npothercommand"] = self.NPCommand.get_text()
-		self.frame.np.config.sections["players"]["npformat"] = self.NPFormat.child.get_text()
-		self.frame.np.config.writeConfig()
-		self.quit(None)
-		
-	def DisplayNowPlaying(self, widget, test=None):
+	def GetNP(self, widget, test=None, callback=None):
 		self.title_clear()
 		if self.NP_infopipe.get_active():
 			result = self.xmms()
@@ -404,12 +467,13 @@ class NowPlaying:
 			result = self.rhythmbox()
 		elif self.NP_bmpx.get_active():
 			result = self.bmpx()
+		elif self.NP_exaile.get_active():
+			result = self.exaile()
 		elif self.NP_other.get_active():
 			result = self.other()
 		if not result:
 			return None
 
-		#print self.title
 		title = self.NPFormat.child.get_text()
 		title = title.replace("$t", self.title["title"])
 		title = title.replace("$a", self.title["artist"])
@@ -427,8 +491,38 @@ class NowPlaying:
 			self.label7.set_text(title)
 			return None
 		if title:
-			#print title
+			if callback:
+				callback(title)
 			return title
+		return None
+		
+	def OnNPSave(self, widget):
+		if self.NP_infopipe.get_active():
+			player = "infopipe"
+		elif self.NP_amarok.get_active():
+			player = "amarok"
+		elif self.NP_audacious.get_active():
+			player = "audacious"
+		elif self.NP_mpd.get_active():
+			player = "mpd"
+		#elif self.NP_mp3blaster.get_active():
+		#	player = "mp3blaster"
+		elif self.NP_rhythmbox.get_active():
+			player = "rhythmbox"
+		elif self.NP_bmpx.get_active():
+			player = "bmpx"
+		elif self.NP_exaile.get_active():
+			player = "exaile"
+		elif self.NP_other.get_active():
+			player = "other"
+			
+		self.frame.np.config.sections["players"]["npplayer"] = player
+		self.frame.np.config.sections["players"]["npothercommand"] = self.NPCommand.get_text()
+		self.frame.np.config.sections["players"]["npformat"] = self.NPFormat.child.get_text()
+		self.frame.np.config.writeConfig()
+		self.quit(None)
+		
+
 		
 	
 	def get_custom_widget(self, id, string1, string2, int1, int2):
@@ -436,17 +530,15 @@ class NowPlaying:
 		return w
 	
 	def bmpx(self):
+		if self.bus is None:
+			self.frame.logMessage(_("ERROR: DBus not available:")+" "+"BMPx"+ " "+ _("cannot be contacted"))
+			return
 		try:
-			import dbus
-			import dbus.glib
-			bus = dbus.SessionBus()
-		except Exception, e:
-			print "NowPlaying Error while loading dbus", e	
-		try:
-			bmp_object = bus.get_object('org.beepmediaplayer.bmp', '/Core')
+			bmp_object = self.bus.get_object('org.beepmediaplayer.bmp', '/Core')
 			bmp_iface = dbus.Interface(bmp_object, 'org.beepmediaplayer.bmp')
-		except Exception, e:
-			print "NowPlaying Error while accessing bmpx dbus interface", e
+		except Exception, error:
+			self.frame.logMessage(_("ERROR while accessing the %(program)s DBus interface: %(error)s") % {"program": "BMPx", "error": error})
+			return
 		try:
 			if bmp_iface.GetCurrentSource() == -1:
 				return None
@@ -461,8 +553,8 @@ class NowPlaying:
 			self.title["album"] = metadata["album"]
 			self.title["filename"] = metadata["location"]
 			return 1 
-		except Exception, e:
-			print "NowPlaying Error during RPC", e
+		except Exception, error:
+			self.frame.logMessage(_("ERROR while reading data from the %(program)s DBus interface: %(error)s") % {"program": "BMPx", "error": error})
 			return None
 		
 	
@@ -495,6 +587,47 @@ class NowPlaying:
 		if output == '' or output.startswith("MPD_HOST") or output.startswith("volume: "):
 			return None
 		return output
+	
+	def exaile(self):
+		slist = self.NPFormat.child.get_text()
+		commandlist = []
+		if "$t" in slist:
+			commandlist.append("--get-title")
+		if "$l" in slist:
+			commandlist.append("--get-length")
+		if "$a" in slist:
+			commandlist.append("--get-artist")
+		if "$b" in slist:
+			commandlist.append("--get-album")
+
+		output = self.exaile_command(commandlist)
+		if output is None:
+			return 0
+		if len(output) == len(commandlist):
+			pos = 0
+			for command in commandlist:
+				if command == "--get-title":
+					self.title["title"] = output[pos]
+				elif command == "--get-length":
+					self.title["length"] = output[pos]
+				elif command == "--get-artist":
+					self.title["artist"] = output[pos]
+				elif command == "--get-album":
+					self.title["album"] = output[pos]
+				pos += 1
+		else:
+			return 0
+		return 1
+		
+				
+	def exaile_command(self, commandlist):
+		command = ""
+		for i in commandlist:
+			command += i + " "
+		output = commands.getoutput("exaile %s" % command).split('\n')
+	
+		return output
+		
 	def amarok(self):
 		slist = self.NPFormat.child.get_text()
 		
@@ -553,7 +686,8 @@ class NowPlaying:
 
 	def audacious(self):
 		slist = self.NPFormat.child.get_text()
-		
+		output = ""
+		self.audacious_running = True
 		if "$n" in slist:
 			artist = self.audacious_command('current-song-tuple-data', 'performer')
 			title = self.audacious_command('current-song-tuple-data', 'track_name')
@@ -590,32 +724,32 @@ class NowPlaying:
 		if "$s" in slist:
 			output = self.audacious_command('playback-status')
 			if output: self.title["status"] = output
+		if not self.audacious_running:
+			self.frame.logMessage(_("ERROR: audtool didn't detect a running Audacious session."))
+			return 0
 		return 1
 		
 	def audacious_command(self, command, subcommand = ''):
 		output = commands.getoutput("audtool %s %s" % (command, subcommand)).split('\n')[0]
-		if output == 'call failed':
+		if output.startswith('audtool'):
 			output = None
+			self.audacious_running = False
 		return output
 
 	def mp3blaster(self):
 		return None
 	
 	def rhythmbox(self):
-		try:
-			import dbus
-			import dbus.glib
-			bus = dbus.SessionBus()
-		except Exception, e:
-			print "NowPlaying Error loading dbus", e
+		if self.bus is None:
+			self.frame.logMessage(_("ERROR: DBus not available:")+" "+"Rhythmbox"+ " "+ _("cannot be contacted"))
 			return None
 		try:
-			rbshellobj = bus.get_object('org.gnome.Rhythmbox', '/org/gnome/Rhythmbox/Shell')
+			rbshellobj = self.bus.get_object('org.gnome.Rhythmbox', '/org/gnome/Rhythmbox/Shell')
 			self.rbshell = dbus.Interface(rbshellobj, 'org.gnome.Rhythmbox.Shell')
 			rbplayerobj = bus.get_object('org.gnome.Rhythmbox', '/org/gnome/Rhythmbox/Player')
 			rbplayer = dbus.Interface(rbplayerobj, 'org.gnome.Rhythmbox.Player')
-		except Exception, e:
-			print "NowPlaying Error opening Rhythmbox interface on dbus", e
+		except Exception, error:
+			self.frame.logMessage(_("ERROR while accessing the %(program)s DBus interface: %(error)s") % {"program": "Rhythmbox", "error": error })
 			return None
 
 		try:
@@ -661,20 +795,26 @@ class NowPlaying:
 			else:
 				self.title["nowplaying"] = output 
 			return 1
-		except Exception, e:
-			print "NowPlaying Error running command %s" %  othercommand, e
+		except Exception, error:
+			self.frame.logMessage(_("ERROR: Executing '%(command)s' failed: %(error)s") % {"command": othercommand, "error": error})
 			return None
 	
 	def xmms(self):
 		if not os.path.exists("/tmp/xmms-info"):
+			self.frame.logMessage(_("ERROR: /tmp/xmms-info does not exist. Is the Infopipe plugin installed and is XMMS running?"))
 			return None
 		try:
-			fsock = open("/tmp/xmms-info")
-			do = fsock.readlines()
+			fsock = file("/tmp/xmms-info")
+			do = fsock.read().split("\n")
+			fsock.close()
+		
+			if len(do) == 0:
+				self.frame.logMessage(_("ERROR: /tmp/xmms-info is empty. Is the Infopipe plugin installed and is XMMS running?"))
 			infolist = []
 			for i in do:
-				if i != "":
-					infolist.append( i.strip("\n") )
+				if i == "":
+					continue
+				infolist.append( i )
 			#protocol  = infolist[0][29:]
 			#version = infolist[1][25:]
 			status = infolist[2][8:]
@@ -690,7 +830,7 @@ class NowPlaying:
 			title = infolist[12][7:]
 			filename = infolist[13][7:]
 			self.title = { "title": "",  "artist": "", "comment": "", "year": "", "album": "", "track":"", "nowplaying": title, "length": time, "bitrate": bitrate, "channels": channels, "position": position, "filename": filename, "status": status}
-			fsock.close()
+
 			return 1
 		except:
 			return None
