@@ -1,28 +1,43 @@
-# Copyright (c) 2003-2004 Hyriand. All rights reserved.
+# Copyright (C) 2007 daelstorm. All rights reserved.
 #
-import gtk
-import gobject
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+# Original copyright below
+# Copyright (c) 2003-2004 Hyriand. All rights reserved.
+
 import os
-import pango
+import gtk, gobject, pango
+import sets
 from nicotine_glade import PrivateChatTab
-from utils import AppendLine, IconNotebook, PopupMenu, WriteLog, expand_alias, EncodingsMenu, SaveEncoding, fixpath
+from utils import AppendLine, IconNotebook, PopupMenu, WriteLog, expand_alias, is_alias, EncodingsMenu, SaveEncoding, fixpath
 from chatrooms import GetCompletion
 from pynicotine import slskmessages
-
 from pynicotine.utils import _, version
 
 class PrivateChats(IconNotebook):
 	def __init__(self, frame):
-		IconNotebook.__init__(self, frame.images)
+		ui = frame.np.config.sections["ui"]
+		IconNotebook.__init__(self, frame.images, ui["labelprivate"], ui["tabclosers"])
 		self.popup_enable()
-		self.set_tab_pos(gtk.POS_TOP)
+
 		self.frame = frame
 		self.connected = 1
 		self.users = {}
 		self.connect("switch-page", self.OnSwitchPage)
 
 	def OnSwitchPage(self, notebook, page, page_num, force = 0):
-		if self.frame.notebook1.get_current_page() != 1 and not force:
+		if self.frame.MainNotebook.get_current_page() != 1 and not force:
 			return
 		page = notebook.get_nth_page(page_num)
 		
@@ -32,47 +47,117 @@ class PrivateChats(IconNotebook):
 				# Remove hilite if selected tab belongs to a user in the hilite list
 				if user in self.frame.TrayApp.tray_status["hilites"]["private"]:
 					self.frame.ClearNotification("private", tab.user)
-		
+					
+	def ClearNotifications(self):
+		if self.frame.MainNotebook.get_current_page() != 1:
+			return
+		page = self.get_nth_page( self.get_current_page())
+		for user, tab in self.users.items():
+			if tab.Main == page:
+				# Remove hilite
+				if user in self.frame.TrayApp.tray_status["hilites"]["private"]:
+					self.frame.ClearNotification("private", tab.user)
+					
+	def Focused(self, page, focused):
+		if not focused:
+			return
+		for user, tab in self.users.items():
+			if tab.Main == page:
+				if user in self.frame.TrayApp.tray_status["hilites"]["private"]:
+					self.frame.ClearNotification("private", tab.user)
+					
 	def GetUserStatus(self, msg):
-		if self.users.has_key(msg.user):
+		if msg.user in self.users:
 			tab = self.users[msg.user]
 			status = [_("Offline"), _("Away"), _("Online")][msg.status]
 			self.set_text(tab.Main, "%s (%s)" % (msg.user[:15], status))
 			tab.GetUserStatus(msg.status)
 
 	def SendMessage(self, user, text = None, direction = None):
-		if not self.users.has_key(user):
+		if user not in self.users:
 			tab = PrivateChat(self, user)
 			self.users[user] = tab
 			self.append_page(tab.Main, user, tab.OnClose)
 			self.frame.np.queue.put(slskmessages.AddUser(user))
-			self.frame.np.queue.put(slskmessages.GetUserStatus(user))
+
 		if direction:
 			if self.get_current_page() != self.page_num(self.users[user].Main):
 				self.set_current_page(self.page_num(self.users[user].Main))
 		if text is not None:
 			self.users[user].SendMessage(text)
 	
+	def TabPopup(self, user):
+		if user not in self.users:
+			return
+		popup = PopupMenu(self.frame)
+		popup.setup(
+			("#" + _("Show IP a_ddress"), popup.OnShowIPaddress, gtk.STOCK_NETWORK),
+			("#" + _("Get user i_nfo"), popup.OnGetUserInfo, gtk.STOCK_DIALOG_INFO),
+			("#" + _("Brow_se files"), popup.OnBrowseUser, gtk.STOCK_HARDDISK),
+			("#" + _("Gi_ve privileges"), popup.OnGivePrivileges, gtk.STOCK_JUMP_TO),
+			("#" + _("Client Version"), popup.OnVersion, gtk.STOCK_ABOUT ),
+			("", None),
+			("$" + _("Add user to list"), popup.OnAddToList),
+			("$" + _("Ban this user"), popup.OnBanUser),
+			("$" + _("Ignore this user"), popup.OnIgnoreUser),
+			("", None),
+			("#" + _("Detach this tab"), self.users[user].Detach, gtk.STOCK_REDO),
+			("#" + _("Close this tab"), self.users[user].OnClose, gtk.STOCK_CLOSE),
+		)
+		popup.set_user(user)
+		
+		items = popup.get_children()
+		
+		items[6].set_active(user in [i[0] for i in self.frame.np.config.sections["server"]["userlist"]])
+		items[7].set_active(user in self.frame.np.config.sections["server"]["banlist"])
+		items[8].set_active(user in self.frame.np.config.sections["server"]["ignorelist"])
+	
+		return popup
+		
+	def on_tab_click(self, widget, event, child):
+		if event.type == gtk.gdk.BUTTON_PRESS:
+			n = self.page_num(child)
+			page = self.get_nth_page(n)
+			username =  [user for user, tab in self.users.items() if tab.Main is page][0]
+			if event.button == 3:
+				menu = self.TabPopup(username)
+				menu.popup(None, None, None, event.button, event.time)
+				return True
+	
+			return False
+		return False
+			
 	def ShowMessage(self, msg, text, status=None):
 		if msg.user in self.frame.np.config.sections["server"]["ignorelist"]:
 			return
 
+		
+		self.SendMessage(msg.user, None)
+		chat = self.users[msg.user]
+		self.request_changed(chat.Main)
+		
+		if self.is_tab_detached(chat.Main):
+			# Only show notifications if window is not focused, and don't
+			# highlight main window
+			if not self.is_detached_tab_focused(chat.Main):
+				self.frame.Notification("private", msg.user)
+		else:
+			# If tab isn't detached
+			# Hilight main private chats Label 
+			self.frame.RequestIcon(self.frame.PrivateChatTabLabel)
+			# Show notifications if the private chats notebook isn't selected,
+			# the tab is not selected, or the main window isn't mapped
+			if self.get_current_page() != self.page_num(chat.Main) or self.frame.MainNotebook.get_current_page() != 1 or not self.frame.is_mapped:
+				self.frame.Notification("private", msg.user)
+
+		# SEND CLIENT VERSION to user if the following string is sent
 		ctcpversion = 0
 		if text == "\x01VERSION\x01":
 			ctcpversion = 1
 			text = "CTCP VERSION"
-		self.SendMessage(msg.user, None)
-		tab = self.users[msg.user]
-		tab.ShowMessage(text, status)
+		self.users[msg.user].ShowMessage(text, status, msg.timestamp)	
 		if ctcpversion and self.frame.np.config.sections["server"]["ctcpmsgs"] == 0:
 			self.SendMessage(msg.user, "Nicotine-Plus %s" % version)
-		self.request_changed(tab.Main)
-		self.frame.RequestIcon(self.frame.PrivateChatTabLabel)
-		if self.get_current_page() != self.page_num(self.users[msg.user].Main) or self.frame.notebook1.get_current_page() != 1:
-			self.frame.Notification("private", msg.user)
-		
-		#else:
-			#self.frame.MainWindow.set_urgency_hint(False)
 
 	def UpdateColours(self):
 		for chat in self.users.values():
@@ -101,7 +186,13 @@ class PrivateChats(IconNotebook):
 		self.connected = 0
 		for user in self.users:
 			self.users[user].ConnClose()
+			tab = self.users[user]
+			self.set_text(tab.Main, "%s (%s)" % (user[:15], _("Offline")))
+			tab.GetUserStatus(0)
 			
+	def UpdateCompletions(self):
+		for user in self.users.values():
+			user.GetCompletionList()
 			
 class PrivateChat(PrivateChatTab):
 	def __init__(self, chats, user):
@@ -110,10 +201,13 @@ class PrivateChat(PrivateChatTab):
 		self.user = user
 		self.chats = chats
 		self.frame = chats.frame
+		if not self.frame.np.config.sections["ui"]["tooltips"]:
+			self.tooltips.disable()
 		self.logfile = None
 		self.autoreplied = 0
 		self.offlinemessage = 0
 		self.status = -1
+		self.clist = []
 		self.Elist = {}
 		self.encoding, m = EncodingsMenu(self.frame.np, "userencoding", user)
 		self.EncodingStore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
@@ -137,6 +231,15 @@ class PrivateChat(PrivateChatTab):
 			self.hbox5.pack_start(self.ChatLine)
 			self.hbox5.reorder_child(self.ChatLine, 0)
 			
+		completion = gtk.EntryCompletion()
+		self.ChatLine.set_completion(completion)
+		liststore = gtk.ListStore(gobject.TYPE_STRING)
+		completion.set_model(liststore)
+		
+		completion.set_text_column(0)
+		completion.set_match_func(self.frame.EntryCompletionFindMatch, self.ChatLine)
+		completion.connect("match-selected", self.frame.EntryCompletionFoundMatch, self.ChatLine)
+		
 		self.Log.set_active(self.frame.np.config.sections["logging"]["privatechat"])
 
 		if self.frame.translux:
@@ -175,42 +278,65 @@ class PrivateChat(PrivateChatTab):
 		)
 		popup.set_user(user)
 		self.ChatScroll.connect("button_press_event", self.OnPopupMenu)
+		self.ChatScroll.connect("key_press_event", self.OnPopupMenu)
 
 		self.UpdateColours()
+		self.GetCompletionList()
 		
+		self.ReadPrivateLog()
+		
+	def ReadPrivateLog(self):
 		# Read log file
-		log = os.path.join(self.frame.np.config.sections["logging"]["logsdir"], fixpath(self.user.replace(os.sep, "-")) + ".log")
+		config = self.frame.np.config.sections
+		log = os.path.join(config["logging"]["privatelogsdir"], fixpath(self.user.replace(os.sep, "-")) + ".log")
+		try:
+			lines = int(config["logging"]["readprivatelines"])
+		except:
+			lines = 15
 		try:
 			f = open(log, "r")
 			d = f.read()
 			f.close()
 			s = d.split("\n")
-			for l in s[-8:-1]:
-				AppendLine(self.ChatScroll, l + "\n", self.tag_hilite, "", username=self.user, usertag=self.tag_hilite)
+			for l in s[- lines :-1]:
+				AppendLine(self.ChatScroll, l + "\n", self.tag_hilite, timestamp_format="", username=self.user, usertag=self.tag_hilite)
 				
 		except IOError, e:
 			pass
-
+		gobject.idle_add(self.frame.ScrollBottom, self.ChatScroll.get_parent())
 		
 	def destroy(self):
 		if self.frame.translux:
 			self.frame.translux.unsubscribe(self.tlux_chat)
-		self.Main.destroy()
+		for i in self.__dict__.keys():
+			del self.__dict__[i]
 		
 	def Login(self):
-		AppendLine(self.ChatScroll, _("--- reconnected ---"), self.tag_hilite, "%c")
+		timestamp_format=self.frame.np.config.sections["logging"]["private_timestamp"]
+		AppendLine(self.ChatScroll, _("--- reconnected ---"), self.tag_hilite, timestamp_format=timestamp_format)
 		self.ChangeColours()
+		
 	def ConnClose(self):
-		AppendLine(self.ChatScroll, _("--- disconnected ---"), self.tag_hilite, "%c")
+		timestamp_format=self.frame.np.config.sections["logging"]["private_timestamp"]
+		AppendLine(self.ChatScroll, _("--- disconnected ---"), self.tag_hilite, timestamp_format=timestamp_format)
+		self.status = -1
 		self.offlinemessage = 0
 		self.ChangeColours()
-	def OnPopupMenu(self, widget, event):
-		if event.button != 3:
-			return
 		
-		self.popup_menu.popup(None, None, None, event.button, event.time)
-		self.ChatScroll.emit_stop_by_name("button_press_event")
-		return True
+	def OnPopupMenu(self, widget, event):
+		if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
+
+			self.popup_menu.popup(None, None, None, event.button, event.time)
+			self.ChatScroll.emit_stop_by_name("button_press_event")
+			return True
+		elif event.type == gtk.gdk.KEY_PRESS:
+
+			if event.keyval == gtk.gdk.keyval_from_name("Menu"):
+				self.popup_menu.popup(None, None, None, 0, 0)
+				self.ChatScroll.emit_stop_by_name("key_press_event")
+				return True
+
+		return False
 	
 	def OnPopupMenuUser(self, widget):
 		items = self.popup_menu_user.get_children()
@@ -239,26 +365,30 @@ class PrivateChat(PrivateChatTab):
 	def OnClearChatLog(self, widget):
 		self.ChatScroll.get_buffer().set_text("")
 	
-	def ShowMessage(self, text, status=None):
+	def ShowMessage(self, text, status=None, timestamp=None):
 		if text[:4] == "/me ":
-			line = "* %s %s" % (self.user, text[4:])
+			line = "* %s %s" % (self.user, self.frame.CensorChat(text[4:]))
+			speech = line[2:]
 			tag = self.tag_me
 		else:
-			line = "[%s] %s" % (self.user, text)
+			line = "[%s] %s" % (self.user, self.frame.CensorChat(text))
+			speech = self.frame.CensorChat(text)
 			tag = self.tag_remote
 		line = self.frame.np.decode(line, self.encoding)
-		AppendLine(self.ChatScroll, line, tag, "%c", username=self.user, usertag=self.tag_my_username)
+		timestamp_format=self.frame.np.config.sections["logging"]["private_timestamp"]
+		AppendLine(self.ChatScroll, line, tag, timestamp=timestamp, timestamp_format=timestamp_format, username=self.user, usertag=self.tag_username)
 		if self.Log.get_active():
-			self.logfile = WriteLog(self.logfile, self.frame.np.config.sections["logging"]["logsdir"], self.user, line)
+			self.logfile = WriteLog(self.logfile, self.frame.np.config.sections["logging"]["privatelogsdir"], self.user, line)
 		
 		autoreply = self.frame.np.config.sections["server"]["autoreply"]
 		if self.frame.away and not self.autoreplied and autoreply:
 			self.SendMessage("[Auto-Message] %s" % autoreply)
 			self.autoreplied = 1
 		if status and not self.offlinemessage:
-			AppendLine(self.ChatScroll, _("* Message(s) sent while you were offline."), self.tag_hilite, "%c")
-			self.offlinemessage = 1
 			
+			AppendLine(self.ChatScroll, _("* Message(s) sent while you were offline."), self.tag_hilite, timestamp_format=timestamp_format)
+			self.offlinemessage = 1
+		self.frame.new_tts(self.frame.np.config.sections["ui"]["speechprivate"] %{"user":self.frame.tts_clean(self.user), "message": self.frame.tts_clean(speech)} )
 
 	def SendMessage(self, text):
 		my_username = self.frame.np.config.sections["server"]["login"]
@@ -275,29 +405,42 @@ class PrivateChat(PrivateChatTab):
 			tag = self.tag_local
 			usertag = self.tag_my_username
 			message = "[" + my_username + "] " + self.frame.np.decode(line, self.encoding)
-		
-		AppendLine(self.ChatScroll, message, tag, "%c", username=my_username, usertag=usertag)
+		timestamp_format=self.frame.np.config.sections["logging"]["private_timestamp"]
+		AppendLine(self.ChatScroll, message, tag, timestamp_format=timestamp_format, username=my_username, usertag=usertag)
 		if self.Log.get_active():
-			self.logfile = WriteLog(self.logfile, self.frame.np.config.sections["logging"]["logsdir"], self.user, message)
+			self.logfile = WriteLog(self.logfile, self.frame.np.config.sections["logging"]["privatelogsdir"], self.user, message)
 		
 		if self.PeerPrivateMessages.get_active():
 			# not in the soulseek protocol
-			self.frame.np.ProcessRequestToPeer(self.user, slskmessages.PMessageUser(None, my_username, text))
+			self.frame.np.ProcessRequestToPeer(self.user, slskmessages.PMessageUser(None, my_username, self.frame.AutoReplace(text)))
 		else:
-			self.frame.np.queue.put(slskmessages.MessageUser(self.user, text))
+			self.frame.np.queue.put(slskmessages.MessageUser(self.user, self.frame.AutoReplace(text)))
 			
 	CMDS = ["/alias ", "/unalias ", "/whois ", "/browse ", "/ip ", "/pm ", "/msg ", "/search ", "/usearch ", "/rsearch ",
-		"/bsearch ", "/add ", "/buddy ", "/rem ", "/unbuddy ", "/ban ", "/ignore ", "/unban ", "/unignore ", "/clear", "/quit", "/rescan", "/nsa", "/info", "/ctcpversion", "/join"]
+		"/bsearch ", "/add ", "/buddy ", "/rem ", "/unbuddy ", "/ban ", "/ignore ", "/unban ", "/unignore ", "/clear", "/quit", "/exit", "/rescan", "/nsa", "/info", "/ctcpversion", "/join"]
 
+		
+	def threadAlias(self, alias):
+		text = expand_alias(self.frame.np.config.aliases, alias)
+		if not text:
+			return
+		if text[:2] == "//":
+			text = text[1:]
+		self.frame.np.queue.put(slskmessages.SayChatroom(self.room, self.frame.AutoReplace(text)))
+
+			
 	def OnEnter(self, widget):
 		text = self.frame.np.encode(widget.get_text(), self.encoding)
-		result = expand_alias(self.frame.np.config.aliases, text)
-		if result is not None:
-			text = result
+
 		if not text:
 			widget.set_text("")
 			return
-		
+		if is_alias(self.frame.np.config.aliases, text):
+			import thread
+			thread.start_new_thread(self.threadAlias, (text,))
+			widget.set_text("")
+			return
+
 		s = text.split(" ", 1)
 		cmd = s[0]
 		if len(s) == 2 and s[1]:
@@ -308,8 +451,14 @@ class PrivateChat(PrivateChatTab):
 
 		if cmd in ("/alias", "/al"):
 			AppendLine(self.ChatScroll, self.frame.np.config.AddAlias(realargs), None, "")
+			if self.frame.np.config.sections["words"]["aliases"]:
+				self.frame.chatrooms.roomsctrl.UpdateCompletions()
+				self.frame.privatechats.UpdateCompletions()
 		elif cmd in ("/unalias", "/un"):
 			AppendLine(self.ChatScroll, self.frame.np.config.Unalias(realargs), None, "")
+			if self.frame.np.config.sections["words"]["aliases"]:
+				self.frame.chatrooms.roomsctrl.UpdateCompletions()
+				self.frame.privatechats.UpdateCompletions()
 		elif cmd in ["/join", "/j"]:
 			self.frame.np.queue.put(slskmessages.JoinRoom(args))
 		elif cmd in ["/w", "/whois", "/info"]:
@@ -342,19 +491,19 @@ class PrivateChat(PrivateChatTab):
 				self.frame.privatechats.SendMessage(user, msg)
 		elif cmd in ["/s", "/search"]:
 			if realargs:
-				self.frame.searches.DoSearch(realargs, 0)
+				self.frame.Searches.DoSearch(realargs, 0)
 				self.frame.OnSearch(None)
 		elif cmd in ["/us", "/usearch"]:
 			if realargs:
-				self.frame.searches.DoSearch(realargs, 3, [self.user])
+				self.frame.Searches.DoSearch(realargs, 3, [self.user])
 				self.frame.OnSearch(None)
 		elif cmd in ["/rs", "/rsearch"]:
 			if realargs:
-				self.frame.searches.DoSearch(realargs, 1)
+				self.frame.Searches.DoSearch(realargs, 1)
 				self.frame.OnSearch(None)
 		elif cmd in ["/bs", "/bsearch"]:
 			if realargs:
-				self.frame.searches.DoSearch(realargs, 2)
+				self.frame.Searches.DoSearch(realargs, 2)
 				self.frame.OnSearch(None)
 		elif cmd in ["/ad", "/add", "/buddy"]:
 			if args:
@@ -381,14 +530,17 @@ class PrivateChat(PrivateChatTab):
 			self.ChatScroll.get_buffer().set_text("")
 		elif cmd in ["/a", "/away"]:
 			self.frame.OnAway(None)
-		elif cmd in ["/q", "/quit"]:
+		elif cmd in ["/q", "/quit", "/exit"]:
 			self.frame.OnExit(None)
+			return
 		elif cmd in ["/c", "/close"]:
 			self.OnClose(None)
 		elif cmd == "/now":
-			np = self.frame.now.DisplayNowPlaying(None)
-			if np:
-				self.SendMessage(np)
+			self.NowPlayingThread()
+		elif cmd == "/detach":
+			self.Detach()
+		elif cmd == "/attach":
+			self.Attach()
 		elif cmd == "/rescan":
 			self.frame.OnRescan()
 		elif cmd and cmd[:1] == "/" and cmd != "/me" and cmd[:2] != "//":
@@ -402,6 +554,17 @@ class PrivateChat(PrivateChatTab):
 				widget.set_text("")
 			return
 		widget.set_text("")
+		
+	def Attach(self, widget=None):
+		self.chats.attach_tab(self.Main)
+		gobject.idle_add(self.frame.ScrollBottom, self.ChatScroll.get_parent())
+		
+	def Detach(self, widget=None):
+		self.chats.detach_tab(self.Main, _("Nicotine+ Private Chat: %s (%s)") % (self.user, [_("Offline"), _("Away"), _("Online")][self.status]))
+		gobject.idle_add(self.frame.ScrollBottom, self.ChatScroll.get_parent())
+		
+	def NowPlayingThread(self):
+		np = self.frame.now.DisplayNowPlaying(None, 0, self.SendMessage)
 
 	def makecolour(self, buffer, colour):
 		color = self.frame.np.config.sections["ui"][colour]
@@ -425,20 +588,21 @@ class PrivateChat(PrivateChatTab):
 		self.tag_local = self.makecolour(buffer, "chatlocal")
 		self.tag_me = self.makecolour(buffer, "chatme")
 		self.tag_hilite = self.makecolour(buffer, "chathilite")
-		if self.status == 1:
+		if self.status == 1 and self.frame.np.config.sections["ui"]["showaway"]:
 			statuscolor = "useraway"
-		elif self.status == 2:
+		elif self.status == 2 or not self.frame.np.config.sections["ui"]["showaway"] and self.status == 1:
 			statuscolor = "useronline"
 		else:
 			statuscolor = "useroffline"
+		self.tag_username = self.makecolour(buffer, statuscolor)
 		if self.chats.connected:
-			if self.frame.away:
+			if self.frame.away and self.frame.np.config.sections["ui"]["showaway"]:
 				self.tag_my_username = self.makecolour(buffer, "useraway")
 			else:
 				self.tag_my_username = self.makecolour(buffer, "useronline")
 		else:
 			self.tag_my_username = self.makecolour(buffer, "useroffline")
-		self.tag_username = self.makecolour(buffer, statuscolor)
+
 		usernamestyle = self.frame.np.config.sections["ui"]["usernamestyle"]
 		if usernamestyle == "bold":
 			self.tag_username.set_property("weight",  pango.WEIGHT_BOLD)
@@ -460,9 +624,11 @@ class PrivateChat(PrivateChatTab):
 			self.tag_my_username.set_property("underline", pango.UNDERLINE_NONE)
 		self.frame.SetTextBG(self.ChatScroll)
 		self.frame.SetTextBG(self.ChatLine)
+		self.frame.SetTextBG(self.Log)
+		self.frame.SetTextBG(self.PeerPrivateMessages)
 		
 	def changecolour(self, tag, colour):
-		if self.frame.np.config.sections["ui"].has_key(colour):
+		if colour in self.frame.np.config.sections["ui"]:
 			color = self.frame.np.config.sections["ui"][colour]
 		else:
 			color = ""
@@ -500,16 +666,17 @@ class PrivateChat(PrivateChatTab):
 		self.changecolour(self.tag_hilite, "chathilite")
 		color = self.getUserStatusColor(self.status)
 		self.changecolour(self.tag_username, color)
-		self.frame.SetTextBG(self.ChatScroll)
-		self.frame.SetTextBG(self.ChatLine)
-		
 		if self.chats.connected:
-			if self.frame.away:
+			if self.frame.away and self.frame.np.config.sections["ui"]["showaway"]:
 				self.changecolour(self.tag_my_username, "useraway")
 			else:
 				self.changecolour(self.tag_my_username, "useronline")
 		else:
 			self.changecolour(self.tag_my_username, "useroffline")
+		self.frame.SetTextBG(self.ChatScroll)
+		self.frame.SetTextBG(self.ChatLine)
+
+			
 	def getUserStatusColor(self, status):
 		if status == 1:
 			color = "useraway"
@@ -517,21 +684,21 @@ class PrivateChat(PrivateChatTab):
 			color = "useronline"
 		else:
 			color = "useroffline"
+		if not self.frame.np.config.sections["ui"]["showaway"] and color == "useraway":
+			color = "useronline"
 		return color
 		
 	def GetUserStatus(self, status):
 		if status == self.status:
 			return
-		
-		
+
 		self.status = status
-
 		color = self.getUserStatusColor(self.status)
-
 		self.changecolour(self.tag_username, color)
-		
-		#line = "* " + ["User %s is offline", "User %s is away", "User %s is online"][status] % self.user
-		#AppendLine(self.ChatScroll, line, self.tag_hilite, "%c")
+		statusword = [_("Offline"), _("Away"), _("Online")][status]
+		title = _("Nicotine+ Private Chat: %s (%s)") % (self.user, statusword)
+		self.chats.set_detached_tab_title(self.Main, title)
+
 	
 	def OnClose(self, widget):
 		if self.logfile is not None:
@@ -540,6 +707,43 @@ class PrivateChat(PrivateChatTab):
 		self.chats.RemoveTab(self)
 		self.destroy()
 
+
+	def GetCompletionList(self, ix=0, text=""):
+		config = self.frame.np.config.sections["words"]
+		completion = self.ChatLine.get_completion()
+		completion.set_popup_single_match(not config["onematch"])
+		completion.set_minimum_key_length(config["characters"])
+		
+		liststore = completion.get_model()
+		liststore.clear()
+		self.clist = []
+		
+		if not config["tab"]:
+			return
+		
+		clist = [self.user, self.frame.np.config.sections["server"]["login"], "nicotine"]
+		if config["buddies"]:
+			clist += [i[0] for i in self.frame.userlist.userlist]
+		if config["aliases"]:
+			clist += ["/"+k for k in self.frame.np.config.aliases.keys()]
+		if config["commands"]:
+			clist += self.CMDS
+		if config["roomnames"]:
+			clist += self.frame.chatrooms.roomsctrl.rooms
+		
+		# no duplicates
+		clist = list(sets.Set(clist))
+		clist.sort(key=str.lower)
+		
+		completion.set_popup_completion(False)
+		if config["dropdown"]:
+			for word in clist:
+				liststore.append([word])
+		completion.set_popup_completion(True)
+		self.clist = clist
+	
+
+		
 	def OnKeyPress(self, widget, event):
 		if event.keyval == gtk.gdk.keyval_from_name("Prior"):
 			scrolled = self.ChatScroll.get_parent()
@@ -555,12 +759,17 @@ class PrivateChat(PrivateChatTab):
 			adj.set_value(new)
 		if event.keyval != gtk.gdk.keyval_from_name("Tab"):
 			return False
+		config = self.frame.np.config.sections["words"]
+		if not config["tab"]:
+			return False
 		ix = widget.get_position()
 		text = widget.get_text()[:ix].split(" ")[-1]
-		list = [self.user] + [i[0] for i in self.frame.userlist.userlist] + ["nicotine"]
-		if ix == len(text) and text[:1] == "/":
-			list += ["/"+k for k in self.frame.np.config.aliases.keys()] + self.CMDS
-		completion, single = GetCompletion(text, list)
+		
+		if widget.get_text()[:1] == "/":
+			self.GetCompletionList(ix, text)
+			
+		completion, single = GetCompletion(text, self.clist)
+		
 		if completion:
 			widget.insert_text(completion, ix)
 			widget.set_position(ix + len(completion))
