@@ -1,5 +1,21 @@
-# Copyright (c) 2003-2004 Hyriand. All rights reserved.
+# Copyright (C) 2007 daelstorm. All rights reserved.
 #
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+# Original copyright below
+# Copyright (c) 2003-2004 Hyriand. All rights reserved.
+
 import gtk
 import gobject
 import pango
@@ -9,7 +25,7 @@ import os, sys
 import string
 import re
 import types
-
+import urllib
 from struct import unpack
 import imghdr
 
@@ -18,7 +34,7 @@ from pynicotine.utils import _
 
 DECIMALSEP = ""
 
-URL_RE = re.compile("(\\w+\\://[\\w\\.].+?)[\\s\\(\\)]|(www\\.\\w+\\.\\w+.*?)[\\s\\(\\)]|(mailto\\:\\w.+?)[\\s\\(\\)]")
+URL_RE = re.compile("(\\w+\\://.+?)[\\s\\(\\)]|(www\\.\\w+\\.\\w+.*?)[\\s\\(\\)]|(mailto\\:\\w.+?)[\\s\\(\\)]")
 PROTOCOL_HANDLERS = {}
 CATCH_URLS = 0
 HUMANIZE_URLS = 0
@@ -56,7 +72,34 @@ def popupWarning(parent, title, warning, icon=None):
 		dlg.destroy()
 		
 	return 0
+def numfmt(value):
+	v = str(float(value)) + '0000'
+	i = v.index('.')
+	if i < 4:
+		return v[:5]
+	else:
+		return v[:i+2]
 	
+def HumanizeBytes(size):
+	if size is None:
+		return None
+
+	try:
+		s = int(size)
+		if s >= 1000*1024*1024:
+			r = _("%s GB") % numfmt(float(s) / 1073741824.0 )
+		elif s >= 1000*1024:
+			r = _("%s MB") % numfmt(float(s) / 1048576.0)
+		elif s >= 1000:
+			r = _("%s KB") % numfmt(float(s) / 1024.0)
+		else:
+			r = _("%s  B") % numfmt(float(s) )
+		return r
+	except Exception, e:
+		Output(e)
+		return size
+	return str(size)
+		
 def recode(s):
 	try:
 		return s.decode(locale.nl_langinfo(locale.CODESET), "replace").encode("utf-8", "replace")
@@ -97,7 +140,7 @@ def InitialiseColumns(treeview, *args):
 			column = gtk.TreeViewColumn(c[0], renderer, active = i)
 		else:
 			renderer = gtk.CellRendererPixbuf()
-			column = gtk.TreeViewColumn(c[0], renderer, pixbuf = 0)
+			column = gtk.TreeViewColumn(c[0], renderer, pixbuf = i)
 		if c[1] == -1:
 			column.set_resizable(False)
 			column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
@@ -161,7 +204,7 @@ def UrlEvent(tag, widget, event, iter, url):
 		if url[:4] == "www.":
 			url = "http://" + url
 		protocol = url[:url.find(":")]
-		if PROTOCOL_HANDLERS.has_key(protocol):
+		if protocol in PROTOCOL_HANDLERS:
 			if PROTOCOL_HANDLERS[protocol].__class__ is types.MethodType:
 				PROTOCOL_HANDLERS[protocol](url.strip())
 			else:
@@ -170,18 +213,27 @@ def UrlEvent(tag, widget, event, iter, url):
 				os.system( cmd)
 		else:
 			try:
-				import gnome.vfs
-				gnome.url_show(url)
-			except:
-				pass
+				import gnomevfs
+			except Exception, e:
+				try:
+					import gnome.vfs
+				except:
+					pass
+				else:
+					gnome.url_show(url)
+			else:
+				try:
+					gnomevfs.url_show(url)
+				except:
+					pass
+				
 	tag.last_event_type = event.type
 
-def AppendLine(textview, line, tag = None, timestamp = "%H:%M:%S", username=None, usertag=None, scroll=True):
+def AppendLine(textview, line, tag = None, timestamp = None, showstamp=True, timestamp_format = "%H:%M:%S", username=None, usertag=None, scroll=True):
 	def _makeurltag(buffer, tag, url):
 		props = {}
-		if tag is not None:
-			color = tag.get_property("foreground_gdk")
-			props["foreground_gdk"] = color
+
+		props["foreground_gdk"] = gtk.gdk.color_parse(NICOTINE.np.config.sections["ui"]["urlcolor"])
 		props["underline"] = pango.UNDERLINE_SINGLE
 		tag = buffer.create_tag(**props)
 		tag.last_event_type = -1
@@ -190,6 +242,7 @@ def AppendLine(textview, line, tag = None, timestamp = "%H:%M:%S", username=None
 
 	def _append(buffer, text, tag):
 		iter = buffer.get_end_iter()
+
 		if tag is not None:
 			buffer.insert_with_tags(iter, text, tag)
 		else:
@@ -204,10 +257,16 @@ def AppendLine(textview, line, tag = None, timestamp = "%H:%M:%S", username=None
 	ME = 0
 	if line.startswith("* "):
 		ME = 1
-	if timestamp:
-		line = "%s %s\n" % (recode(time.strftime(timestamp)), line)
-
+	if NICOTINE.np.config.sections["logging"]["timestamps"] and showstamp:
+		if timestamp_format and not timestamp:
+			line = "%s %s\n" % (recode(time.strftime(timestamp_format)), line)
+		elif timestamp_format and timestamp:
+			line = "%s %s\n" % (recode(time.strftime(timestamp_format, time.localtime(timestamp))), line)
+	else:
+		line += "\n"
+		
 	match = URL_RE.search(line)
+	# Highlight urls, if found and tag them
 	while CATCH_URLS and match:
 		start = line[:match.start()]
 		url = match.group()[:-1]
@@ -229,7 +288,9 @@ def AppendLine(textview, line, tag = None, timestamp = "%H:%M:%S", username=None
 		else:
 			_append(buffer, start, tag)
 		if url.startswith("slsk://") and HUMANIZE_URLS:
-			url = url.replace("%20", " ")
+			
+			url = urllib.url2pathname( url)
+
 		_append(buffer, url, urltag)
 		match = URL_RE.search(line)
 	
@@ -260,35 +321,159 @@ def AppendLine(textview, line, tag = None, timestamp = "%H:%M:%S", username=None
 	return linenr
 	
 class ImageLabel(gtk.HBox):
-	def __init__(self, label = "", image = None, onclose = None):
+	def __init__(self, label = "", image = None, onclose = None, closebutton = False, angle = 0, show_image = True):
 		gtk.HBox.__init__(self)
-		self.set_spacing(2)
-
+		
+		self.closebutton = closebutton
+		self.angle = angle
+		self._show_image = show_image
+		self.notify = 0
+		
 		self._entered = 0
 		self._pressed = 0
+		
 		self.onclose = onclose
-
-		self.label = gtk.Label(label)
+		
+		self.label = gtk.Label()
+		self.text = label
+		if NICOTINE.np.config.sections["ui"]["tab_colors"]:
+			color = NICOTINE.np.config.sections["ui"]["tab_default"]
+		else:
+			color = ""
+		if not color:
+			self.label.set_text("%s" % self.text)
+		else:
+			self.label.set_markup("<span foreground=\"%s\">%s</span>" % (color, self.text.replace("<", "&lt;").replace(">", "&gt;")))
+			
 		self.label.set_alignment(0.0, 0.50)
-		self.pack_start(self.label, True, True)
+		self.label.set_angle(angle)
 		self.label.show()
 
 		self.image = gtk.Image()
 		self.set_image(image)
-		self.pack_start(self.image, False, False)
-		self.image.show()
+		
+		if self._show_image:
+			self.image.show()
 
-		if onclose is not None:
-			self.button = gtk.Button()
-			img = gtk.Image()
-			img.set_from_stock(gtk.STOCK_CLOSE, gtk.ICON_SIZE_MENU)
-			self.button.add(img)
-			self.button.connect("clicked", onclose)
-			self.button.set_relief(gtk.RELIEF_NONE)
-			self.button.set_size_request(18,18)
-			self.pack_start(self.button, False, False)
-			self.button.show_all()
 
+		self._pack_children()
+		self._order_children()
+		
+	def _pack_children(self):
+		self.set_spacing(0)
+		if "Box" in self.__dict__:
+			for widget in self.Box.get_children():
+				self.Box.remove(widget)
+			self.remove(self.Box)
+			self.Box.destroy()
+			del self.Box
+
+		if self.angle in ( 90, -90):
+			self.Box = gtk.VBox()
+		else:
+			self.angle = 0
+			self.Box = gtk.HBox()
+
+		self.Box.set_spacing(2)
+		self.add(self.Box)
+		self.Box.show()
+		
+		self.Box.pack_start(self.label, True, True)
+		self.Box.pack_start(self.image, False, False)
+		if self.closebutton and self.onclose is not None:
+			self._add_close_button()
+			
+	def _order_children(self):
+
+		if self.angle == 90:
+			if "button" in self.__dict__ and self.closebutton != 0:
+				self.Box.reorder_child(self.button, 0)
+				self.Box.reorder_child(self.image, 1)
+				self.Box.reorder_child(self.label, 2)
+			else:
+				self.Box.reorder_child(self.image, 0)
+				self.Box.reorder_child(self.label, 1)
+		else:
+			self.Box.reorder_child(self.label, 0)
+			self.Box.reorder_child(self.image, 1)
+			if "button" in self.__dict__ and self.closebutton != 0:
+				self.Box.reorder_child(self.button, 2)
+
+				
+	def set_onclose(self, closebutton):
+		self.closebutton = closebutton
+		
+		if self.closebutton:
+			self._add_close_button()
+		else:
+			self._remove_close_button()
+		self._order_children()
+		
+	def show_image(self, show = True):
+		self._show_image = show
+		
+		if self._show_image:
+			self.image.show()
+		else:
+			self.image.hide()
+		
+	def set_angle(self, angle):
+		self.angle = angle
+		self.label.set_angle(self.angle)
+		self._remove_close_button()
+
+		self._pack_children()
+		self._order_children()
+		
+	def _add_close_button(self):
+		if "button" in self.__dict__:
+			return
+		self.button = gtk.Button()
+		img = gtk.Image()
+		img.set_from_stock(gtk.STOCK_CLOSE, gtk.ICON_SIZE_MENU)
+		self.button.add(img)
+		if self.onclose is not None:
+			self.button.connect("clicked", self.onclose)
+		self.button.set_relief(gtk.RELIEF_NONE)
+
+		self.button.show_all()
+		self.Box.pack_start(self.button, False, False)
+		
+
+	def _remove_close_button(self):
+		if "button" not in self.__dict__:
+			return
+		self.Box.remove(self.button)
+		self.button.destroy()
+		del self.button
+		
+	def set_text_color(self, notify = None, text = None):
+		if notify is None:
+			notify = self.notify
+		else:
+			self.notify = notify
+		if NICOTINE.np.config.sections["ui"]["tab_colors"]:
+			if notify == 1:
+				color = NICOTINE.np.config.sections["ui"]["tab_changed"]
+			elif notify == 2:
+				color = NICOTINE.np.config.sections["ui"]["tab_hilite"]
+			else:
+				color = NICOTINE.np.config.sections["ui"]["tab_default"]
+				
+			try: 
+				gtk.gdk.color_parse(color)
+			except:
+				color = ""
+		else:
+			color = ""
+		if text is not None:
+			self.text = text
+		if not color:
+			self.label.set_text("%s" % self.text)
+			
+		else:
+			self.label.set_markup("<span foreground=\"%s\">%s</span>" % (color, self.text.replace("<", "&lt;").replace(">", "&gt;")))
+			
 	def set_image(self, img):
 		self.img = img
 		self.image.set_from_pixbuf(img)
@@ -297,21 +482,49 @@ class ImageLabel(gtk.HBox):
 		return self.img
 		
 	def set_text(self, lbl):
-		self.label.set_text(lbl)
+		if NICOTINE.np.config.sections["ui"]["tab_colors"]:
+			self.set_text_color( notify = None, text = lbl)
+		else:
+			self.text = lbl
+			self.label.set_text(lbl)
 		
 	def get_text(self):
 		return self.label.get_text()
 	
 class IconNotebook(gtk.Notebook):
-	def __init__(self, images):
-		self.tabclosers = 0
+	def __init__(self, images, angle = 0, tabclosers = False, show_image = True):
+		self.tabclosers = tabclosers
 		gtk.Notebook.__init__(self)
 		self.images = images
+		self._show_image = show_image
 		self.pages = []
+		self.detached_tabs = []
 		self.connect("switch-page", self.dismiss_icon)
 		self.connect("key_press_event", self.OnKeyPress)
 		self.set_scrollable(True)
+		self.angle = angle
 		
+	def set_tab_closers(self, closers):
+
+		self.tabclosers = closers
+		for data in self.pages:
+			page, label_tab, status, label_tab_menu = data
+			label_tab.set_onclose(self.tabclosers)
+			
+	def show_images(self, show_image = True):
+		self._show_image = show_image
+		for data in self.pages:
+			page, label_tab, status, label_tab_menu = data
+			label_tab.show_image(self._show_image)
+		
+	def set_tab_angle(self, angle):
+		if angle == self.angle:
+			return
+		self.angle = angle
+		for data in self.pages:
+			page, label_tab, status, label_tab_menu = data
+			label_tab.set_angle(angle)
+			
 	def OnKeyPress(self, widget, event):
 		if event.state & (gtk.gdk.MOD1_MASK | gtk.gdk.CONTROL_MASK) != gtk.gdk.MOD1_MASK:
 			return False
@@ -324,19 +537,149 @@ class IconNotebook(gtk.Notebook):
 		widget.emit_stop_by_name("key_press_event")
 		return True
 	
-	def append_page(self, page, label, onclose = None):
-		if not self.tabclosers:
-			onclose = None
-		l = ImageLabel(label, self.images["empty"], onclose)
-		l2 = ImageLabel(label, self.images["empty"])
-		self.pages.append([page, l, 0, l2])
-		gtk.Notebook.append_page_menu(self, page, l, l2)
+	def append_page(self, page, label, onclose = None, angle = 0):
+		self.set_tab_angle(angle)
+		closebutton = self.tabclosers
+
+		label_tab = ImageLabel(label, self.images["empty"], onclose, closebutton = closebutton, angle = angle, show_image = self._show_image)
+		# menu for all tabs
+		label_tab_menu = ImageLabel(label, self.images["empty"])
+		self.pages.append([page, label_tab, 0, label_tab_menu])
+		eventbox = gtk.EventBox()
+		label_tab.show()
+		eventbox.add(label_tab)
+		eventbox.show()
+		eventbox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
+		eventbox.connect('button_press_event', self.on_tab_click, page)
+		gtk.Notebook.append_page_menu(self, page, eventbox, label_tab_menu)
+		try:
+			self.set_tab_reorderable(page, True)
+			#self.set_tab_detachable(page, True)
+		except:
+			# Old PyGTK2
+			pass
+		
+	def OnTabWindowDestroy(self, widget, page):
+		if self.is_tab_detached(page):
+			self.attach_tab(page, destroying=True)
+			
+	def detach_tab(self, page, title=_("Nicotine+")):
+		label = None
+		if self.is_tab_detached(page):
+			return
+		for i in self.pages[:]:
+			if i[0] == page:
+				pagewidget, label_tab, status, label_tab_menu = i
+				label = label_tab.label.get_text()
+				label_tab.get_parent().remove(label_tab)
+				
+				break
+
+		if label is None:
+			return
+		for i in self.detached_tabs:
+			if i[0] == label or i[1] is page:
+				return
+		gtk.Notebook.remove_page(self, self.page_num(page))
+			
+		window = gtk.Window()
+		window.set_title(title)
+		#window.add_accel_group(self.accel_group)
+		window.set_icon(NICOTINE.images["n"])
+		window.resize(600, 400)
+		vbox = gtk.VBox(False, spacing=5)
+		vbox.set_border_width(5)
+		vbox.pack_start(page)
+		vbox.show()
+		window.add(vbox)
+		window.connect("destroy", self.OnTabWindowDestroy, page)
+		window.connect("focus_in_event", self.OnFocusIn)
+		window.connect("focus_out_event", self.OnFocusOut)
+		
+		self.detached_tabs.append([page, label, window, False])
+		window.show()
+		
+	def OnFocusIn(self, widget, event):
+		widget.set_icon(NICOTINE.images["n"])
+		for item in self.detached_tabs:
+			if item[2] == widget:
+				item[3] = True
+				self.Focused(item[0], True)
+				
+	def OnFocusOut(self, widget, event):
+		for item in self.detached_tabs:
+			if item[2] == widget:
+				item[3] = False
+				self.Focused(item[0], False)
+
+	def Focused(self, page, focused):
+		pass
+	
+	def attach_tab(self, page, destroying=False):
+		pagewidget = label_tab = label_tab_menu = label = None
+
+		for item in self.detached_tabs:
+			if item[0] is page:
+				label = item[1]
+				window = item[2]
+				break
+		if label is None or window is None:
+			return
+		for i in self.pages[:]:
+			if i[0] == page:
+				label = i[1].label.get_text()
+				pagewidget, label_tab, status, label_tab_menu = i
+				break
+		for i in (pagewidget, label_tab, label_tab_menu, label, status):
+			if i is None:
+				return
+		window.get_child().remove(pagewidget)
+		
+		#self.pages.append([page, label_tab, status, label_tab_menu])
+		eventbox = gtk.EventBox()
+		label_tab.show()
+		eventbox.add(label_tab)
+		eventbox.show()
+		eventbox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
+		eventbox.connect('button_press_event', self.on_tab_click, page)
+		gtk.Notebook.append_page_menu(self, pagewidget, eventbox, label_tab_menu)
 		try:
 			self.set_tab_reorderable(page, True)
 		except:
 			# Old PyGTK2
 			pass
+		self.detached_tabs.remove(item)
+		if not destroying:
+			window.destroy()
 		
+	def is_tab_detached(self, page):
+		for item in self.detached_tabs:
+			if item[0] is page:
+				return True
+		return False
+
+	def is_detached_tab_focused(self, page):
+		for item in self.detached_tabs:
+			if item[0] is page:
+				return item[3]
+		return False
+	
+	def set_detached_icon(self, page, status):
+		image = self.images[("empty", "online", "hilite")[status]]
+		for item in self.detached_tabs:
+			if item[0] is page:
+				window = item[2]
+				window.set_icon(image)
+				
+	def set_detached_tab_title(self, page, title):
+		for item in self.detached_tabs:
+			if item[0] is page:
+				window = item[2]
+				window.set_title(title)
+				
+	def on_tab_click(self, widget, event, child):
+		pass
+	
 	def set_image(self, page, status):
 		image = self.images[("empty", "online", "hilite")[status]]
 		for i in self.pages:
@@ -355,22 +698,44 @@ class IconNotebook(gtk.Notebook):
 				i[1].set_text(label)
 				i[3].set_text(label)
 				return
-				
+	
+	def set_text_colors(self, color = None):
+		for i in self.pages:
+			i[1].set_text_color(color)
+			
+	def set_text_color(self, page, color = None):
+		for i in self.pages:
+			if i[0] == page:
+				i[1].set_text_color(color)
+				return
+			
 	def dismiss_icon(self, notebook, page, page_num):
 		page = self.get_nth_page(page_num)
 		self.set_image(page, 0)
+		
+		self.set_text_color(page, 0)
 
 	def request_hilite(self, page):
+		if self.is_tab_detached(page):
+			if self.is_detached_tab_focused(page):
+				return
+			self.set_detached_icon(page, 2)
 		current = self.get_nth_page(self.get_current_page())
 		if current == page:
 			return
 		self.set_image(page, 2)
+		self.set_text_color(page, 2)
 
 	def request_changed(self, page):
+		if self.is_tab_detached(page):
+			if self.is_detached_tab_focused(page):
+				return
+			self.set_detached_icon(page, 1)
 		current = self.get_nth_page(self.get_current_page())
 		if current == page:
 			return
 		self.set_image(page, 1)
+		self.set_text_color(page, 1)
 	
 	def remove_page(self, page):
 		for i in self.pages[:]:
@@ -379,6 +744,7 @@ class IconNotebook(gtk.Notebook):
 				i[1].destroy()
 				i[3].destroy()
 				self.pages.remove(i)
+				
 				return
 
 class PopupMenu(gtk.Menu):
@@ -387,6 +753,8 @@ class PopupMenu(gtk.Menu):
 		self.frame = frame
 		self.user = None
 		self.useritem = None
+		self.handlers = {}
+		
 	def setup(self, *items):
 		for item in items:
 			if item[0] == "":
@@ -398,19 +766,21 @@ class PopupMenu(gtk.Menu):
 			elif item[0] == 1:
 				menuitem = gtk.MenuItem(item[1])
 				menuitem.set_submenu(item[2])
-				if item[3] is not None:
-					menuitem.connect("activate", item[3])
+				if len(item) == 5 and item[4] is not None and item[3] is not None:
+					self.handlers[menuitem] = menuitem.connect("activate", item[3], item[4])
+				elif item[3] is not None:
+					self.handlers[menuitem] = menuitem.connect("activate", item[3])
 			elif item[0] == "USERMENU":
 				menuitem = gtk.MenuItem(item[1])
 				menuitem.set_submenu(item[2])
 				if item[3] is not None:
-					menuitem.connect("activate", item[3])
+					self.handlers[menuitem] = menuitem.connect("activate", item[3])
 				self.useritem = menuitem
 			elif item[0] == 2:
 				menuitem = gtk.ImageMenuItem(item[1])
 				menuitem.set_submenu(item[2])
 				if item[3] is not None:
-					menuitem.connect("activate", item[3])
+					self.handlers[menuitem] = menuitem.connect("activate", item[3])
 				img = gtk.image_new_from_stock(item[4], gtk.ICON_SIZE_MENU)
 				menuitem.set_image(img)
 			else:
@@ -428,10 +798,22 @@ class PopupMenu(gtk.Menu):
 				else:
 					menuitem = gtk.MenuItem(item[0])
 				if item[1] is not None:
-					menuitem.connect("activate", item[1])
+					self.handlers[menuitem] = menuitem.connect("activate", item[1])
 			self.append(menuitem)
 			menuitem.show()
 		return self
+				
+	def clear(self):
+		for widget in self.handlers.keys():
+			widget.disconnect(self.handlers[widget])
+			
+		self.handlers.clear()
+		for widget in self.get_children():
+			self.remove(widget)
+			widget.destroy()
+		if self.useritem is not None:
+			self.useritem.destroy()
+			self.useritem = None
 
 	def set_user(self, user):
 		self.user = user
@@ -440,10 +822,15 @@ class PopupMenu(gtk.Menu):
 
 	def get_user(self):
 		return self.user
+	
+	def OnSearchUser(self, widget):
+		self.frame.SearchMethod.set_active_iter(self.frame.searchmethods[_("User")])
+		self.frame.UserSearchCombo.child.set_text(self.user)
+		self.frame.MainNotebook.set_current_page(4)
 		
 	def OnSendMessage(self, widget):
 		self.frame.privatechats.SendMessage(self.user, None, 1)
-		self.frame.notebook1.set_current_page(1)
+		self.frame.MainNotebook.set_current_page(1)
 	
 	def OnShowIPaddress(self, widget):
 		self.frame.np.queue.put(slskmessages.GetPeerAddress(self.user))
@@ -476,7 +863,13 @@ class PopupMenu(gtk.Menu):
 		self.frame.privatechats.SendMessage(self.user, "\x01VERSION\x01")
 		
 	def OnGivePrivileges(self, widget):
-		text = InputDialog(None, _("Give privileges"), _("Give how many days of global privileges to this user?") )
+		self.frame.np.queue.put(slskmessages.CheckPrivileges())
+		if self.frame.np.privileges_left is None:
+			days = _("Unknown")
+		else:
+			days = self.frame.np.privileges_left / 60 / 60 / 24
+			
+		text = InputDialog(self.frame.MainWindow, _("Give privileges")+" "+_("to %(user)s") %{"user": self.user}, _("Give how many days of global privileges to this user?") + " ("+ _("%(days)s days left") %{'days':days} +")" )
 		if text:
 			try:
 				days = int(text)
@@ -612,7 +1005,8 @@ def WriteLog(logfile, logsdir, fn, msg):
 			os.makedirs(logsdir)
 		logfile = open(os.path.join(logsdir, fixpath(fn.replace(os.sep, "-")) + ".log"), 'a', 0)
 		os.umask(oldumask)
-	logfile.write("%s %s\n" % (recode(time.strftime("%c")), msg))
+	
+	logfile.write("%s %s\n" % (recode(time.strftime(NICOTINE.np.config.sections["logging"]["log_timestamp"])), msg))
 	logfile.flush()
 	return logfile
 		
@@ -643,7 +1037,17 @@ def Humanize(number):
 		part, number = number[-3:], number[:-3]
 		ret = "%s%s%s" % (part, fashion, ret)
 	return neg + ret[:-1]
- 
+
+def is_alias(aliases, cmd):
+	if not cmd:
+		return False
+	if cmd[0] != "/":
+		return False
+	cmd = cmd[1:].split(" ")
+	if cmd[0] in aliases:
+		return True
+	return False
+
 def expand_alias(aliases, cmd):
 	def getpart(line):
 		if line[0] != "(":
@@ -663,72 +1067,97 @@ def expand_alias(aliases, cmd):
 			ix = ix + 1
 		return ""
 
-	if not cmd:
+	if not is_alias(aliases, cmd):
 		return None
-	if cmd[0] != "/":
-		return None
-	cmd = cmd[1:].split(" ")
-	if not aliases.has_key(cmd[0]):
-		return None
-	alias = aliases[cmd[0]]
-	ret = ""
-	i = 0
-	while i < len(alias):
-		if alias[i:i+2] == "$(":
-			arg=getpart(alias[i+1:])
-			if not arg:
-				ret = ret + "$"
-				i = i + 1
-				continue
-			i = i + len(arg) + 3
-			args = arg.split("=",1)
-			if len(args) > 1:
-				default = args[1]
-			else:
-				default = ""
-			args = args[0].split(":")
-			if len(args) == 1:
-				first = last = int(args[0])
-			else:
-				if args[0]:
-					first = int(args[0])
+	try:
+		cmd = cmd[1:].split(" ")
+		alias = aliases[cmd[0]]
+		ret = ""
+		i = 0
+		while i < len(alias):
+			if alias[i:i+2] == "$(":
+				arg=getpart(alias[i+1:])
+				if not arg:
+					ret = ret + "$"
+					i = i + 1
+					continue
+				i = i + len(arg) + 3
+				args = arg.split("=",1)
+				if len(args) > 1:
+					default = args[1]
 				else:
-					first = 1
-				if args[1]:
-					last = int(args[1])
+					default = ""
+				args = args[0].split(":")
+				if len(args) == 1:
+					first = last = int(args[0])
 				else:
-					last = len(cmd)
-			v = string.join(cmd[first:last+1])
-			if not v: v = default
-			ret = ret + v
-		elif alias[i:i+2] == "|(":
-			arg = getpart(alias[i+1:])
-			if not arg:
-				ret = ret + "|"
+					if args[0]:
+						first = int(args[0])
+					else:
+						first = 1
+					if args[1]:
+						last = int(args[1])
+					else:
+						last = len(cmd)
+				v = string.join(cmd[first:last+1])
+				if not v: v = default
+				ret = ret + v
+			elif alias[i:i+2] == "|(":
+				arg = getpart(alias[i+1:])
+				if not arg:
+					ret = ret + "|"
+					i = i + 1
+					continue
+				i = i + len(arg) + 3
+				for j in range(len(cmd)-1, -1, -1):
+					arg = arg.replace("$%i" % j, cmd[j])
+				arg = arg.replace("$@", string.join(cmd[1:], " "))
+				version = sys.version_info
+				if version[0] == 3 or (version[0] >= 2 and version[1] >= 4):
+					import subprocess
+	
+					p = subprocess.Popen(arg, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE, close_fds=(not sys.platform.startswith("win")))
+					exit = p.wait()
+
+					(stdout, stdin) = (p.stdout, p.stdin)
+					v = stdout.read().split("\n")
+					r = ""
+					for l in v:
+						l = l.strip()
+						if l:
+							r = r + l + "\n"
+					ret = ret + r.strip()
+					stdin.close()
+					stdout.close()
+				
+				else:
+					stdin, stdout = os.popen2(arg)
+					v = stdout.read().split("\n")
+					r = ""
+					for l in v:
+						l = l.strip()
+						if l:
+							r = r + l + "\n"
+					ret = ret + r.strip()
+					stdin.close()
+					stdout.close()
+					try:
+						os.wait()
+					except OSError, error:
+						pass
+					except Exception, error:
+						pass
+			else:
+				ret = ret + alias[i]
 				i = i + 1
-				continue
-			i = i + len(arg) + 3
-			for j in range(len(cmd)-1, -1, -1):
-				arg = arg.replace("$%i" % j, cmd[j])
-			arg = arg.replace("$@", string.join(cmd[1:], " "))
-			stdin, stdout = os.popen2(arg)
-			v = stdout.read().split("\n")
-			r = ""
-			for l in v:
-				l = l.strip()
-				if l:
-					r = r + l + "\n"
-			ret = ret + r.strip()
-			stdin.close()
-			stdout.close()
-			os.wait()
-		else:
-			ret = ret + alias[i]
-			i = i + 1
-	return ret
+		return ret
+	except Exception, error:
+		print error
+		pass
+	return ""
 
 def EncodingsMenu(np, section = None, entry = None):
-	if section and entry and np.config.sections["server"][section].has_key(entry):
+	if section and entry and entry in np.config.sections["server"][section]:
 		encoding = np.config.sections["server"][section][entry]
 	else:
 		encoding = np.config.sections["server"]["enc"]
@@ -742,7 +1171,7 @@ def EncodingsMenu(np, section = None, entry = None):
 def SaveEncoding(np, section, entry, encoding):
 	if encoding != np.config.sections["server"]["enc"]:
 		np.config.sections["server"][section][entry] = encoding
-	elif np.config.sections["server"][section].has_key(entry):
+	elif entry in np.config.sections["server"][section]:
 		del np.config.sections["server"][section][entry]
 	np.config.writeConfig()
 	
@@ -777,11 +1206,11 @@ class ImportWinSlskConfig:
 		if self.BuddyList:
 			# Getting userlist
 			users = self.get_basic_config(self.winpath('hotlist.cfg'))
-		
+
 			for i in users:
-				if not self.is_in_user_list(i,self.config.sections["server"]["userlist"]):
+				if not self.is_in_user_list(i, self.config.sections["server"]["userlist"]):
 					#print [i,'']
-					self.config.sections["server"]["userlist"].append([i,''])
+					self.config.sections["server"]["userlist"].append([i, "", 0, 0, 0, "", ""])
 		
 		if self.Login:
 			# Get login and password
@@ -827,10 +1256,23 @@ class ImportWinSlskConfig:
 		return 1
   
 	def check_slskdir(self):
-		# we check if the file queue2.cfg exists under the slsk dir
-		queue2path = os.path.join(self.Path, 'queue2.cfg')
 		
-		if not os.access(queue2path, os.F_OK):
+		# we check if the file queue2.cfg exists under the slsk dir
+		if self.Queue:
+			path = self.winpath('queue2.cfg')
+		elif self.BuddyList:
+			path = self.winpath('hotlist.cfg')
+		elif self.Login:
+			path = self.winpath('login.cfg')
+		elif self.Rooms:
+			path = self.winpath('chatrooms.cfg')
+		elif self.BanList:
+			path = self.winpath('dlbans.cfg')
+		elif self.IgnoreList:
+			path = self.winpath('ignores.cfg')
+		elif self.UserInfo:
+			path = self.winpath('userinfo.cfg')
+		if not os.access(path, os.F_OK):
 			return 0
 		return 1
 
@@ -857,7 +1299,7 @@ class ImportWinSlskConfig:
 		
 		The file can contain some unknown stuff at the end."""
 		
-		infile = open(fname, 'r')
+		infile = open(fname, 'rb')
 		intsize = 4
 		downloads = []
 		
@@ -889,7 +1331,7 @@ class ImportWinSlskConfig:
 		- followed by some undetermined bytes"""
 		
 		
-		infile = open(fname, 'r')
+		infile = open(fname, 'rb')
 		intsize = 4
 		downloads = []
 		
@@ -917,7 +1359,7 @@ class ImportWinSlskConfig:
 		
 		- The length of the username, followed by the username"""
 		
-		infile = open(fname, 'r')
+		infile = open(fname, 'rb')
 		intsize = 4
 		users = []
 		
@@ -940,7 +1382,7 @@ class ImportWinSlskConfig:
 		- one byte, an indication for an image. 0 is no image, 1 is image
 		- an int with the image size, followed by the image data """
 		
-		infile = open(fname, 'r')
+		infile = open(fname, 'rb')
 		intsize = 4
 		
 		imgdata = None
@@ -971,7 +1413,7 @@ class ImportWinSlskConfig:
 			return ""
 		
 		fname = os.path.abspath(self.config.filename + '-userinfo.' + extension)
-		outfile = open(fname, "w")
+		outfile = open(fname, "wb")
 		outfile.write(imgdata)
 		outfile.close()
 		print "Wrote image to \"%s\"" % (fname)
